package LudumDare23.Planes;

import LudumDare23.*;
import com.ixee.datastruct.Coordinate;
import com.ixee.GraphIX.*;
import java.util.*;
import com.ixee.io.*;

public class PlaneStarter extends Plane{


	LinkedList<Integer> actions;
	long lastframechangetime;
	long activelastframechangetime;
	glEAnim currentAnim;
	glEAnim colonizeAnim;

	glEImage colonizeImage;

	Hookshot dragmearound = null;

	boolean colonizing;

	boolean engineMakesSounds = true;

	double colonizeRate = 3000;
	double pitch;

	int width = 0;
	int height = 0;

	long lastupdatetime;


	String activeAnimName = "StarterActiveAnim";
	String baseName = "PlaneStarter";


	public PlaneStarter(){
		ambientSound = "Propeller";
		maxlife = 5;
		life = 5;
		currentVelocity = 10;
		acceleration = 70;
		minVelocity = 3;
		maxVelocity = 70;
		maxAltitude = 0.8;
		minAltitude = 0.4;
		ascendRate = 20;

		actions = new LinkedList<Integer>();
		reference = Game.clock();
		orientation = 0;
		parent = null;
		currentAnim = null;
		lastframechangetime = reference.time();
		lastupdatetime = reference.time();
		setAnim(Game.getAnim(baseImage()));
		setActiveAnim(Game.getAnim(activeAnimName()));
	}
	public void setColonizeAnim(glEAnim e){
		colonizeAnim = e;
	}
	public void update(){
		double delta = reference.delta(lastupdatetime)/1000.0;
		boolean turnOnSound = false;
		boolean turnOffSound = false;
		/*
			Do some math.
		*/
		if(parent == null){
			stdio.debugAndLog(this, "This is me screaming very loudly because the parent to this is null. NOOOOOOOOOO!!");
		}else{
			pitch = 0;
			while(actions.size() > 0){
				Integer currentAction = actions.pollFirst();
				if(dragmearound == null){
					switch(currentAction){
						case ACTION_IDLING:
							pitch = 0;
							break;
						case ACTION_ACCELERATING:
							currentVelocity = Math.min(maxVelocity, currentVelocity+delta*acceleration);
							break;
						case ACTION_DECELERATING:
							currentVelocity = Math.max(minVelocity, currentVelocity-delta*acceleration);
							break;
						case HOOKSHOT:
							//Test to see if we -CAN- hookshot
							if(dragmearound == null){
								Planet dest = Game.hookshotDest(this);
								if(dest != null){
								turnOffSound = true;
									dragmearound = new Hookshot(this, dest);
								/*
									If we can, figure out where it hits what we're hookshotting
									Probably use a separate class to draw the hookshot and manage dragging us to them
									//Ask the game if we're pointing at a planet.
								*/
								}
							}
							break;
						case ACTION_ASCENDING:
							altitude = Math.min(parent.atmospheresize()*maxAltitude, altitude+delta*ascendRate);
							pitch = Math.toRadians(15);
							break;
						case ACTION_DESCENDING:
							altitude = Math.max(parent.atmospheresize()*minAltitude, altitude-delta*ascendRate);
							pitch = Math.toRadians(-15);
							break;
						case ACTIVE:
							startColonizer();
							break;

					}
				}else{
					colonizing = false;
				}
			}
		}
		if(dragmearound != null){
			engineMakesSounds = false;
			//Then we're hookshotting
			if(dragmearound.timeleft()<=0){
				dragmearound.dest().give(this);
				Coordinate newDelta = getPosition().sub(parent.getPosition());
				orientation = Math.atan2(newDelta.getY(), newDelta.getX());
				dragmearound = null;
				turnOnSound = true;
			}
		}else{
			engineMakesSounds = true;
			orientation -= delta*currentVelocity/(altitude);
			if(orientation < -Math.PI){
				orientation += 2*Math.PI;
			}else if(orientation > Math.PI){
				orientation -= 2*Math.PI;
			}
			if(colonizing && Game.playerPlane() == this){
				/*
					Dump some people on the parent planet
				*/
				//System.out.println(orientation/(2*Math.PI));
				//System.out.println("Giving "+delta*colonizeRate+" people");
				parent.addPeopleToSection(delta*colonizeRate, orientation/(2*Math.PI));
			}
		}
		if(Game.playerPlane() == this){
			if(turnOnSound){
				Game.playSound(ambientSound);
			}else if(turnOffSound){
				Game.stopSound(ambientSound);
			}
		}
		lastupdatetime = reference.time();
	}
	public boolean hookshotting(){
		return (dragmearound != null);
	}
	public String baseImage(){
		return baseName;
	}
	public String[] animationNames(){
		return new String[]{"PlaneStarter_moving", "PlaneStarter_colonizing"};
	}
	public void draw(glEEngine e){
		if(colonizing){
			if(reference.delta(activelastframechangetime) > colonizeAnim.getMsPerFrame()){
				colonizeAnim.next();
				activelastframechangetime = reference.time();
			}
			e.drawImage(colonizeAnim.current(),  getPosition().sub(Game.camera.getlocation()), ((float)orientation+(float)pitch-Math.PI/2)/2);
		}
		if(dragmearound != null){
			dragmearound.draw(e);
		}
		if(reference.delta(lastframechangetime) > currentAnim.getMsPerFrame()){
			currentAnim.next();
			lastframechangetime = reference.time();
		}
		e.drawImage(currentAnim.current(), getPosition().sub(Game.camera.getlocation()), ((float)orientation+(float)pitch-Math.PI/2)/2);
	}
	public void putInPlanet(Planet p){
		if(parent != null){
			parent.remove(this);
		}
		parent = p;
		altitude = parent.atmospheresize()*minAltitude;
	}
	public void setAnim(glEAnim e){
		currentAnim = e.copy();
		height = currentAnim.current().getSize().intY();
		width = currentAnim.current().getSize().intX();
		currentAnim.setMsPerFrame(80);
	}
	public void setActiveAnim(glEAnim e){
		colonizeAnim = e.copy();
		colonizeAnim.setMsPerFrame(250);
	}
	public String activeAnimName(){
		return activeAnimName;
	}
	public void pushAction(int action){
		actions.push(action);
	}
	public void startColonizer(){
		colonizing = true;
	}
	public int height(){
		return height;
	}
	public int width(){
		return width;
	}
	public Coordinate getPosition(){
		Coordinate returnable;
		if(dragmearound == null){
			return super.getPosition();
		}else{
			return dragmearound.myposition();
		}
	}
	public void collide(Entity e){
		if(e instanceof Resource){
			if(this == Game.playerPlane){
				Game.givePlayer((Resource)e);
			}
		}
	}
}

