/*
	Planets are divided up into 15 slivers that have a value to show how populated they are.

	As the population passes a certain threshold, that sliver becomes considered actually populated, and life begins there.
*/

package LudumDare23;

import com.ixee.phys.*;
import com.ixee.datastruct.Coordinate;
import com.ixee.GraphIX.*;
import java.awt.*;
import com.ixee.io.stdio;
import java.util.*;

public class Planet{
	GameTime reference;

	glEImage colonyImage;

	ArrayList<Entity> atmosphericContents;
	double[] populations;
	int numSections = 10;
	double thresholdOfPopulation = 10000;
	double atmosphereradius;
	double groundradius;

	Coordinate position;
	Coordinate velocity;
	Coordinate lastDelta;

	long lastupdatetime;

	public Planet(GameTime t){
		atmosphericContents = new ArrayList<Entity>();
		populations = new double[numSections];
		atmosphereradius = 180;
		groundradius = 60;
		position = new Coordinate(0,0);
		velocity = new Coordinate(0,0);
		lastDelta = new Coordinate(0,0);
		reference = t;
		lastupdatetime = t.time();

		colonyImage = Game.getImage("Colony");
	}
	public double atmospheresize(){
		return atmosphereradius;
	}
	public double groundradius(){
		return groundradius;
	}
	public void draw(glEEngine e){
		e.drawCircle(position.sub(Game.camera.getlocation()), (float)atmosphereradius, 9, new Color(132, 207, 211));
		e.drawCircle(position.sub(Game.camera.getlocation()), (float)groundradius, 9, new Color(186, 126, 90));
		for(int i=0;i<populations.length;i++){
		//	System.out.println(populations[i]);
			if(populations[i]>thresholdOfPopulation){
				double angle = Math.toRadians(360*i/populations.length+Math.PI+Math.PI*2/populations.length);
				e.drawImage(colonyImage, position.sub(Game.camera.getlocation()).sub(new Coordinate(groundradius*Math.cos(angle), groundradius*Math.sin(angle))), (float)(angle+Math.PI/2)/2);
			}
		}
		for(int i=0;i<atmosphericContents.size();i++){
			Game.draw(atmosphericContents.get(i));
		}
	}
	public void give(Entity t){
		atmosphericContents.add(t);
		t.putInPlanet(this);
	}
	public void remove(Entity t){
		atmosphericContents.remove(t);
	}
	public Entity get(int t){
		if(t>-1 && t<atmosphericContents.size()){
			return atmosphericContents.get(t);
		}else{
			return null;
		}
	}
	public int numEntities(){
		return atmosphericContents.size();
	}
	public void update(){
		double delta = reference.delta(lastupdatetime)/1000;
		lastDelta = velocity.mult(delta);
		position = position.add(lastDelta);
		for(int i=0;i<atmosphericContents.size();i++){
			Entity e = atmosphericContents.get(i);
			atmosphericContents.get(i).update();
			if(e.parent() != this){
				i--;
			}else if(!atmosphericContents.get(i).alive()){
				System.out.println("Removing "+atmosphericContents.get(i));
				Game.stopSound(atmosphericContents.get(i).getAmbientSound());
				atmosphericContents.remove(i);
				i--;
			}
		}
		for(int i=0;i<atmosphericContents.size();i++){
			Entity testing = atmosphericContents.get(i);
			if(!testing.hookshotting()){
				Coordinate testingDelta = testing.getPosition().sub(getPosition());
				double testingAngle = Math.atan2(testing.width()/2, Math.sqrt(Game.pythag(testingDelta)));

				for(int j=i+1;j<atmosphericContents.size();j++){
					Entity testee = atmosphericContents.get(j);

					if(!testee.hookshotting()){


						Coordinate testeeDelta = testee.getPosition().sub(getPosition());
						double testeeAngle = Math.atan2(testee.width(), Math.sqrt(Game.pythag(testeeDelta)));

						boolean verticalCollision = false;
						boolean horizontalCollision = false;


						//System.out.println("Testing: "+testing.getAltitude()+", "+testing.height()+"; testee: "+testee.getAltitude()+", "+testee.height());

						if(testing.getAltitude()-testing.height()/2 > testee.getAltitude()-testee.height()/2 && testing.getAltitude()-testing.height()/2 < testee.getAltitude()+testee.height()/2){
							verticalCollision = true;
						}else if(testing.getAltitude()+testing.height()/2 > testee.getAltitude()-testee.height()/2 && testing.getAltitude()+testing.height()/2 < testee.getAltitude()+testee.height()/2){
							verticalCollision = true;

						}else if((testing.getAltitude()+testing.height()/2 > testee.getAltitude() + testee.height()/2) && (testing.getAltitude()-testing.height()/2 < testee.getAltitude() - testee.height()/2)){
							verticalCollision = true;

						}else if((testing.getAltitude() + testing.height()/2 < testee.getAltitude() + testee.height() && testing.getAltitude() - testing.height()/2 > testee.getAltitude()-testee.height()/2)){
							verticalCollision = true;

						}

						if(Math.sin(testee.getRotation()-testeeAngle) < Math.sin(testing.getRotation() + testingAngle) && Math.sin(testee.getRotation()-testeeAngle) > Math.sin(testing.getRotation() - testingAngle)){
							horizontalCollision = true;
						}else if(Math.sin(testee.getRotation()+testeeAngle) > Math.sin(testing.getRotation()-testingAngle) && Math.sin(testee.getRotation()+testeeAngle) < Math.sin(testing.getRotation()+testingAngle)){
							horizontalCollision = true;
						}else if((Math.sin(testee.getRotation()-testeeAngle) < Math.sin(testing.getRotation() - testingAngle) && Math.sin(testee.getRotation() + testeeAngle) > Math.sin(testing.getRotation() + testingAngle))){
							horizontalCollision = true;
						}else if((Math.sin(testee.getRotation()-testeeAngle) > Math.sin(testing.getRotation() - testingAngle) && Math.sin(testee.getRotation() + testeeAngle) < Math.sin(testing.getRotation() + testingAngle))){
							horizontalCollision = true;
						}
						if(Math.cos(testee.getRotation()) * Math.cos(testing.getRotation()) < 0 && horizontalCollision == true){
							horizontalCollision = false;
						}
						if(verticalCollision && horizontalCollision){
							testing.collide(testee);
							testee.collide(testing);
							//collision
						}
					}
				}

				/*
					if near plane

					blow it up
				*/
			}
		}
		lastupdatetime = reference.time();
	}
	public Coordinate getPosition(){
		return position;
	}
	public void setPosition(Coordinate c){
		position = c;
	}
	public void setVelocity(Coordinate c){
		velocity = c;
	}
	public void addPeopleToSection(double numPeeps, double percentageAround){
		int sectionNum = (int)Math.floor(populations.length*(percentageAround+0.5));
		if(sectionNum >= 0 && sectionNum < populations.length){
			boolean populated = populations[sectionNum]>=thresholdOfPopulation;
			populations[sectionNum] += numPeeps;
			if(populated == false && populations[sectionNum]>=thresholdOfPopulation){
				Game.playSound("ColonyMade");
				Game.playerMaxAltLevel++;
				Game.playerActionLevel++;
				Game.population++;
			}
		}else{
			stdio.debugAndLog(this, "Somehow we're over a nonexistant section! PrecentageAround = "+percentageAround+", numSections: "+populations.length);
		}
	}
}