package com.ixee.GraphIX;

import com.ixee.ui.ogl.*;
import com.ixee.ui.KeyboardInfo;
import com.ixee.ui.CursorInfo;
import java.awt.image.BufferedImage;
import com.ixee.datastruct.Coordinate;

public class GIXEngine extends Thread{	//Figure this out. I have a feeling this might be more complex than it seems.
						//Does each call in the UI menu have to be synchronized? Hm.
						//Possibly set it up such that the menu is checked, updated, and then the checkable values
						//Are only updated after this is all done?
							//Could introduce extra latenency.
							//Overall, it would run faster though, so the extra latency would be lessened.

	glEEngine graphics;
	CursorInfo cursorInfo;
	KeyboardInfo keyboardInfo;
	MenuStackOGL menus;
	Coordinate mouseLoc;

	public GIXEngine(int x, int y){
		this("Default Name", x, y);
	}
	public GIXEngine(String s, int x, int y){
		graphics = new glEEngine(s, x, y);
		graphics.setMode(glEEngine.game);
		graphics.setOrthPersp(new Coordinate(0,0,1));
		GIXInputWrapper.initInput();
		mouseLoc = null;
		menus = new MenuStackOGL();
		cursorInfo = new CursorInfo(graphics.getSize().mult(0.5), GIXVariables.mouseButtons.length);
		keyboardInfo = new KeyboardInfo(GIXVariables.keyPresses.length);
	}
	public void addMenu(MenuOGL menu){
		menus.addMenu(menu);
	}
	public void renderMenus(){
		/*
			graphics draw stuff
		*/
		menus.draw(graphics);
	}
	public MenuStackOGL menus(){
		return menus;
	}
	public glEImage loadImage(BufferedImage b){
		return graphics.loadImage(b);
	}
	public void update(){
		graphics.update();
		GIXInputWrapper.poll();
		cursorInfo.sync(GIXVariables.mousePosition.sub(graphics.getSize().mult(0.5)), GIXVariables.mouseButtons, GIXVariables.mouseButtonsReleased);
		keyboardInfo.sync(GIXVariables.keyPresses);
		checkMouse();
		graphics.clear();
	}
	public boolean mouseIntercepted(){
		return mouseLoc==null;
	}
	public Coordinate mousePosition(){
		return mouseLoc;
	}
	public void checkMouse(){
		mouseLoc = null;
		boolean intercepted = false;
		intercepted = menus.check(cursorInfo, keyboardInfo);
		if(!intercepted && cursorInfo.clicked(0)){
			mouseLoc = cursorInfo.releaseLoc(0);
			//set up returnable
		}
	}
	public glEEngine graphics(){
		return graphics;
	}
	public KeyboardInfo getKeyboardState(){
		return keyboardInfo;
	}
	public CursorInfo getCursorState(){
		return cursorInfo;
	}
	public Coordinate getSize(){
		return graphics.getSize();
	}
	public double cos(double d){
		return GIXVariables.fastMath.cos(d);
	}
	public double sin(double d){
		return GIXVariables.fastMath.cos(d);
	}
	public boolean closeRequested(){
		return graphics.closeRequested();
	}
}