package com.ixee.GraphIX;

import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL14;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.GLContext;
import org.lwjgl.opengl.AWTGLCanvas;
import org.lwjgl.opengl.DisplayMode;
import org.lwjgl.opengl.GLContext;
import org.lwjgl.opengl.PixelFormat;
import org.lwjgl.opengl.ARBVertexBufferObject;
import org.lwjgl.LWJGLException;
import org.lwjgl.BufferUtils;
//import org.lwjgl.util.glu.*;
//import org.lwjgl.util.glu.Project;
import com.ixee.io.stdio;
import com.ixee.font.*;
import com.ixee.datastruct.Coordinate;

import java.io.File;
import java.nio.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.awt.image.BufferedImage;
import java.awt.image.ComponentColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.DataBufferByte;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.awt.*;
import java.awt.color.*;
import javax.swing.JFrame;

public class glEEngine{
	ArrayList<glESprite> objects = new ArrayList<glESprite>();
	ArrayList<glEEffect> effects = new ArrayList<glEEffect>();
	ArrayList<glESprite> spriteBuffer = new ArrayList<glESprite>();
	ArrayList<glEEffect> effectBuffer = new ArrayList<glEEffect>();
	ArrayList<glESprite> spriteDebuffer = new ArrayList<glESprite>();
	ArrayList<glEEffect> effectDebuffer = new ArrayList<glEEffect>();
	ArrayList<glETexture> textureList = new ArrayList<glETexture>(0);
	ArrayList<glELight> lights = new ArrayList<glELight>(0);
	ArrayList<glECamera> cameras = new ArrayList<glECamera>();		//This list does NOT include the primaryCamera!
	glECamera primaryCam;
	glECamera sreenCam;
	glECamera boundCam;
	glECamera nullCam;
	HashMap<String, glEBackgroundImage> backgroundFrames = new HashMap<String, glEBackgroundImage>();//A hashmap containing all the images used to composite into the background, along with their names.
	DisplayMode[] validModes;
	int currDisplayMode=0;
	private boolean disengageEngine;
	private HashMap<String, glEImage> loadedImages;
	private HashMap<String, glEAnim> loadedAnimations;
	private HashMap<String, Integer> loadedResources;
	private boolean started = false;
	private boolean vboSupported = false;
	FontDataAccelerated loadedFont = null;
	String defaultFont = "./com/ixee/GraphIX/font/defaultFont.gixf";
	int boundTex = 0;
	int numSpritesDrawnThisFrame = 0;

	/*
		Internals

		these are internal values for stuff
	*/
	public static final int UI = 0;
	public static final int game = 1;

	int currentMode = -1;

	/*
		settings!

		drawObjects: this boolean toggles drawing sprites
		drawEffects: this boolean toggles drawing effects

		screenSize: this coordinate defines width/height of the screen.
		cameraLocation: this coordinate defines the camera
		angle: this float defines how the
	*/

	boolean drawObjects = true;
	boolean drawEffects = true;

	boolean texturesPrimed = true;

	boolean fullscreen = false;

	boolean modeSet = false;		//Used in the setMode()/revertMode() calls.

	/*
		These settings affect image quality!
	*/
	int antialiasing = 1;

	int smoothing_none = 0;
	int smoothing_nearestNeighbor = 1;
	int smoothing_linear = 2;
	int smoothing_bilinear = 3;
	int smoothing_trilinear = 4;

	int spriteFilter = FILTER_NEAREST;
	int backgroundFilter = FILTER_NEAREST;
	int animFilter = FILTER_NEAREST;

	public static final int FILTER_LINEAR = 0;
	public static final int FILTER_NEAREST = 1;

	//public static final int STENCIL_SET_1 = 0;
	public static final int STENCIL_SET = 1;
	public static final int STENCIL_EQUAL = 2;

	public static final int ALLOW_ALL = 0;
	public static final int ALLOW_NONE = 1;
	public static final int ALLOW_STENCIL_ONLY = 2;
	public static final int ALLOW_COLOR_ONLY = 3;

	//For binding cameras and such.
	public static final int CAM_PRIMARY = 0;

	boolean bloom = false;

	boolean shadows = false;

/*
	Camera settings
*/
	Coordinate screenSize;
	Coordinate cameraLocation;
	float angle;

	String[] objectDrawOrder = {"Border", "Default", "Lights"};



	Canvas canvas;
	public glEEngine(int width, int height){
		this(new String[]{"default"}, width, height);
	}
	public glEEngine(String title, String[] drawOrder, int width, int height){
		Display.setTitle(title);
		GIXVariables.fastMath = new GIXMath();
	//	try{
		GIXVariables.gamewidth = width;
		GIXVariables.gameheight = height;
		screenSize = new Coordinate(width, height, 0);
		primaryCam = new glECamera(new Coordinate(0,0,0), screenSize, 0);
		nullCam = new glECamera(new Coordinate(0, 0, 1), screenSize, 1);
		bindCam(CAM_PRIMARY);
		//canvas = new Canvas();
		//parent.add(canvas);
		//canvas.setFocusable(true);
		//canvas.setIgnoreRepaint(true);
		//parent.setVisible(true);
		init(width, height);
		stdio.log(this, "Initializing GLEngine: "+GL11.glGetString(GL11.GL_VENDOR)+ " "+GL11.glGetString(GL11.GL_RENDERER) +" "+GL11.glGetString(GL11.GL_VERSION));
		loadFont(defaultFont);

		//Have some backup loading going on here to load a backup font file if the first is nonexistant.
		vboSupported = GLContext.getCapabilities().GL_ARB_vertex_buffer_object;
		//canvas.setSize(width, height);
	//	}catch(LWJGLException e){
	//		e.printStackTrace();
	//	}
		objectDrawOrder = drawOrder;
		disengageEngine = false;
		loadedResources = new HashMap<String, Integer>();
		loadedImages = new HashMap<String, glEImage>();
		loadedAnimations = new HashMap<String, glEAnim>();
	//	GL11.glShadeModel(GL11.GL_SMOOTH);
	}
	public glEEngine(String[] drawOrder, int width, int height){
		this("Game", drawOrder, width, height);
	}
	public glEEngine(String title, int width, int height){
		this(title, new String[]{}, width, height);
	}
	public Canvas getDrawingSurface(){
		return canvas;
	}
	public void loadFont(String path){
		if(new File(path).exists()){
			loadedFont = FontLoader.loadAcceleratedFont(path);
			loadedFont.setImageAddress(load(loadedFont.getImage()));
			stdio.debugAndLog(this, "Loaded font: "+path);
		}else{
			//try{
			//	System.out.println(new File(path).getCanonicalPath());
			//}catch(Exception ioe){}
			stdio.debugAndLog(this, "glEEngine.loadfont: File specified doesn't exist! "+path);
		}
	}
	public glEVBOSpriteData[] createBuffers(glESprite s){
		Integer[] toLoad = s.getSpriteIndices();
		String[] resourceNames = s.getResources();
		glEVBOSpriteData[] returnable = new glEVBOSpriteData[toLoad.length];
		for(int i=0;i<toLoad.length;i++){
			//returnable[i] = vboMap.get(s.getTypeName()+resourceNames[i]);
			if(returnable[i] == null){
				//this resource isn't loaded so let's load it.
				ByteBuffer tempBB = ByteBuffer.allocateDirect(4);
				IntBuffer texIDs = tempBB.asIntBuffer();
				ARBVertexBufferObject.glGenBuffersARB(texIDs);
				int vboIndex = texIDs.get(0);
			}
		}
		return returnable;
	}
	public int[] load(glEEffect toLoadFrom, String[] resources){
		System.out.println("glEEngine.load(effect) does nothing! Fix it!");
		return new int[0];
	}
	public int[] load(glESprite toLoadFrom, String[] resources){
		stdio.log(this, "Engine is loading "+resources.length+" sprites");
		int[] returnable = new int[resources.length];

		String typeName = toLoadFrom.getTypeName();

		for(int i=0;i<resources.length;i++){
			stdio.log(this, "Loading "+typeName+resources[i]);
			BufferedImage ourSprite = toLoadFrom.getImage(resources[i]);
			returnable[i] = load(ourSprite);
			loadedResources.put(typeName+resources[i], returnable[i]);
		}
		Integer[] indices = new Integer[returnable.length];
		for(int i=0;i<indices.length;i++){
			indices[i] = new Integer(returnable[i]);
		}
		toLoadFrom.setSpriteIndices(indices);
		return returnable;
	}
	public glEAnim load(glEAnimArchive archive){
		glEAnim returnable;
		if(!loaded(archive)){
		//	System.out.println("Engine is loading animation "+archive.name());
			glEImage[] images = new glEImage[archive.numFrames()];
			BufferedImage resources;
			for(int i=0;i<images.length;i++){
				if(loadedImages.get(archive.name().concat(Integer.toString(i))) != null){
					images[i] = loadedImages.get(archive.name().concat(Integer.toString(i)));
				}else{
					resources = archive.getImage(i);
					int address = load(resources);
					images[i] = new glEImage(address, new Coordinate(0,0), new Coordinate(resources.getWidth(null), resources.getHeight(null)));
					loadedImages.put(archive.name().concat(Integer.toString(i)), images[i]);
				}
			}
			returnable = new glEAnim(images, archive.getDefaultLoops(), archive.getMsPerFrame());
			loadedAnimations.put(archive.name(), returnable.copy());
		}else{
			returnable = loadedAnimations.get(archive.name()).copy();
		}
		return returnable;
	}
	public void setMode(int mode){
		if(modeSet){
			revertMode();
		}

		switch(mode){
			case UI:
				//make it so the arraylist of drawables is the UI list
				currentMode = mode;
				GL11.glDisable(GL11.GL_STENCIL_TEST);
				GL11.glEnable(GL11.GL_BLEND);
				GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
				GL11.glDisable(GL11.GL_DEPTH_TEST);

				GL11.glMatrixMode(GL11.GL_PROJECTION);
				GL11.glPushMatrix();

				GL11.glOrtho(-screenSize.getX()/2, screenSize.getX()/2, -screenSize.getY()/2, screenSize.getY()/2, -1, 1);

				GL11.glMatrixMode(GL11.GL_MODELVIEW);
				GL11.glPushMatrix();
				GL11.glLoadIdentity();
				modeSet = true;
				break;
			case game:
				currentMode = mode;
				//make the arraylist of drawables is the game list
				GL11.glDisable(GL11.GL_STENCIL_TEST);
				GL11.glEnable(GL11.GL_BLEND);
				GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
				GL11.glDisable(GL11.GL_DEPTH_TEST);

				GL11.glMatrixMode(GL11.GL_PROJECTION);
				GL11.glPushMatrix();

				if(boundCam == null){
					stdio.debugAndLog(this, "Bound camera is set to null. That's not cool. This shouldn't really happen...");
					GL11.glOrtho(-screenSize.getX()/2, screenSize.getX()/2, -screenSize.getY()/2, screenSize.getY()/2, -1, 1);
				}else{
					Coordinate cameraLoc = boundCam.getLocation();
					double dist = cameraLoc.getZ();
					GL11.glOrtho(-screenSize.getX()*dist/2, screenSize.getX()*dist/2, -screenSize.getY()*dist/2, screenSize.getY()*dist/2, -1, 1);
				}
				GL11.glMatrixMode(GL11.GL_MODELVIEW);
				GL11.glPushMatrix();
				GL11.glLoadIdentity();
				modeSet = true;
				break;
		}
		currentMode = mode;
	}
	public void revertMode(){
		GL11.glMatrixMode(GL11.GL_PROJECTION);
		GL11.glPopMatrix();
		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glPopMatrix();
		modeSet = false;
	}
	public void putBackgroundFrame(glEBackgroundImage backgroundImage, String name){
		backgroundFrames.put(name, backgroundImage);
	}
	public void setBackgroundParallax(String backgroundFrame, double parallax){
	//	if(backgroundFrames.get(backgroundFrame) != null){
		backgroundFrames.get(backgroundFrame).setParallaxFactor(parallax);
	//	}
	}
	public void setBackgroundVisibility(String backgroundFrame, boolean visibility){
		if(backgroundFrames.get(backgroundFrame) != null){
			backgroundFrames.get(backgroundFrame).setDrawable(visibility);
		}
	}
	public glEImage loadImage(BufferedImage image){
		int addr = load(image);
		//System.out.println(image);
		glEImage returnable = new glEImage(addr, new Coordinate(0,0,0), new Coordinate(image.getWidth(null), image.getHeight(null), 0));
		return returnable;
	}
	public Integer load(BufferedImage ourSprite){
		ByteBuffer tempBB = ByteBuffer.allocateDirect(4);
		IntBuffer texIDs = tempBB.asIntBuffer();
		GL11.glGenTextures(texIDs);
		int[] address = new int[1];
		texIDs.get(address);
		int returnable = address[0];

		int height = ourSprite.getHeight(null);
		int width = ourSprite.getWidth(null);
		int resultDimension = Math.max(nearPow2(width),nearPow2(height));

		ComponentColorModel glAlphaColorModel = new ComponentColorModel(ColorSpace.getInstance(ColorSpace.CS_sRGB), new int[] {8,8,8,8},true, false, ComponentColorModel.TRANSLUCENT, DataBuffer.TYPE_BYTE);

		WritableRaster raster = Raster.createInterleavedRaster(DataBuffer.TYPE_BYTE,nearPow2(width), nearPow2(height),4,null);
		BufferedImage texImage = new BufferedImage(glAlphaColorModel,raster,false,new Hashtable());

		Graphics g = texImage.getGraphics();
		//g.setColor(new Color(0f,0f,0f,1f));
		//g.fillRect(0,0,resultDimension,resultDimension);
		g.drawImage(ourSprite,0,0,null);
		//System.out.println("Pasting a "+ourSprite.getWidth(null)+", "+ourSprite.getHeight(null)+" image into "+nearPow2(width)+", "+nearPow2(height));
		DataBuffer buf = texImage.getRaster().getDataBuffer();

		DataBufferByte ourBuffer = (DataBufferByte)(buf);


		byte[] data = ourBuffer.getData();
		ByteBuffer imageBuffer = ByteBuffer.allocateDirect(data.length);
		imageBuffer.order(ByteOrder.nativeOrder());
		imageBuffer.put(data);
		imageBuffer.rewind();

		GL11.glBindTexture(GL11.GL_TEXTURE_2D, returnable);
		//GLU.gluBuild2DMipmaps(GL11.GL_TEXTURE_2D, 4, nearPow2(width), nearPow2(height), GL11.GL_RGBA, GL11.GL_UNSIGNED_BYTE, imageBuffer);
		GL11.glTexParameterf(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_NEAREST);
		GL11.glTexParameterf(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_NEAREST);
		//GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL14.GL_BUILD_MIPMAPS, GL11.GL_TRUE);
		//GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_NEAREST);
		//GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_NEAREST);
		GL11.glTexImage2D(GL11.GL_TEXTURE_2D, 0, GL11.GL_RGBA, nearPow2(width), nearPow2(height), 0, GL11.GL_RGBA, GL11.GL_UNSIGNED_BYTE, imageBuffer);

		textureList.add(new glETexture(returnable, width, height));

		returnable = textureList.size();

		return returnable-1;
	}
	private void init(int width, int height){
		try{
			DisplayMode[] allValidModes = Display.getAvailableDisplayModes();
			ArrayList<DisplayMode> modes = new ArrayList<DisplayMode>();
			for(int i=0;i<allValidModes.length;i++){
				if(allValidModes[i].getFrequency() == Display.getDesktopDisplayMode().getFrequency()){
					modes.add(allValidModes[i]);
				}
			}
			validModes = modes.toArray(new DisplayMode[modes.size()]);
			Display.setDisplayMode(new DisplayMode(width,height));
			System.out.println("Creating display.");
			Display.create(new PixelFormat(32, 0, 24, 8, 0));
			System.out.println("Created display.");
			Display.update();
		//	Display.setParent(canvas);
			//GLContext.useContext(canvas);
		}catch(LWJGLException e){
			e.printStackTrace();
		}
	}
	public void resize(int mode, boolean tryFullscreen){
		DisplayMode oldMode = Display.getDisplayMode();
		boolean oldFullscreen = fullscreen;
		try{

				//Commented out and bypassed by using the desktop resolution
			Display.setDisplayMode(validModes[mode]);
			if(validModes[mode].isFullscreenCapable()){
				Display.setFullscreen(tryFullscreen);
			}else{
				Display.setFullscreen(false);
			}

			//DisplayMode desktopMode = Display.getDesktopDisplayMode();
			//Display.setDisplayMode(desktopMode);
			//Display.setFullscreen(true);
			screenSize = new Coordinate(validModes[mode].getWidth(), validModes[mode].getHeight());
			//screenSize = new Coordinate(desktopMode.getWidth(), desktopMode.getHeight());
			GIXVariables.gamewidth = screenSize.intX();
			GIXVariables.gameheight = screenSize.intY();
			setMode(currentMode);
			fullscreen = tryFullscreen;
			GL11.glViewport(0, 0, screenSize.intX(), screenSize.intY());
		}catch(LWJGLException e){
			try{
				Display.setDisplayMode(oldMode);
				fullscreen = oldFullscreen;
				Display.setFullscreen(false);
				e.printStackTrace();
			}catch(LWJGLException e2){
				e2.printStackTrace();
			}
		}
	}
	public void drawBackground(){
		/*
			Ensure vertex/texCoord arrays are enabled.

			Set the coordinate system so that (0, 0) is the center of the universe and the screen spans -screenSize to +screenSize exactly.

			For each background frame...
				parallaxFactor is 1/frameParallaxFactor
				scale = (log2(cameraDist)+parallaxFactor)/parallaxFactor
				locToDrawAt is a per-pane centerpoint to draw backgrounds at;
					locToDrawAt = cameraLocation*-parallaxFactor*0.075

				draw image
		*/
		GL11.glEnableClientState(GL11.GL_VERTEX_ARRAY);
		GL11.glEnableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
		GL11.glEnable(GL11.GL_TEXTURE_2D);
		glEBackgroundImage[] backgroundImages = backgroundFrames.values().toArray(new glEBackgroundImage[backgroundFrames.values().size()]);

		setMode(this.UI);		//This is entirely identical to binding a camera at (0, 0, 1) and setting the mode to game, but doesn't muck up camera state management.

//		setOrthPersp(new Coordinate(0, 0, 1));
		for(int i=0;i<backgroundImages.length;i++){
//			if(backgroundImages[i].isDrawable()){
				Coordinate locToDrawAt = new Coordinate(0,0);
				double parallaxFact;
				if(backgroundImages[i].parallaxFactor() != 0){
					parallaxFact = 1/backgroundImages[i].parallaxFactor();
					//float scale = (float)((1+1/parallaxFact)/(1+boundCam.getLocation().getZ()));
					float scale = (float)((Math.log(boundCam.getLocation().getZ())/Math.log(2)+parallaxFact)/parallaxFact);
					locToDrawAt = boundCam.getLocation().mult(-backgroundImages[i].parallaxFactor()*0.075);
					//System.out.println("Drawing background frame "+i+" at scale "+scale);

					glECamera current = boundCam;
					bindCam(nullCam);
					//System.out.println(
					drawImage(
						backgroundImages[i].getAddress(),
						(float)(backgroundImages[i].getSize().getX()/scale),
						(float)(backgroundImages[i].getSize().getY()/scale),
						(float)(locToDrawAt.getX()),
						(float)(locToDrawAt.getY()), 0
						//(float)(backgroundImages[i].getCenter().getX()+locToDrawAt.getX()),
						//(float)(backgroundImages[i].getCenter().getY()+locToDrawAt.getY()), 0
					);
					bindCam(current);
				}else{
					glECamera current = boundCam;
					bindCam(nullCam);
					drawImage(backgroundImages[i].getAddress(), (float)(backgroundImages[i].getSize().getX()), (float)(backgroundImages[i].getSize().getY()), (float)boundCam.getLocation().getX(), (float)boundCam.getLocation().getY(), 0);

					bindCam(current);
				}
//			}
		}
		GL11.glDisableClientState(GL11.GL_VERTEX_ARRAY);
		GL11.glDisableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
		revertPersp();
	}
	public void end(){
		disengageEngine = true;
		Display.destroy();
	}
	public void setOrthPersp(Coordinate c){
		/*
			Sets up a projection matrix to have the viewport span the whole screen size, but with the appropriate scaling for the camera's position (presumed to be c.)
		*/

		//Actually quite the useless method, isn't this?

		GL11.glMatrixMode(GL11.GL_PROJECTION);
		GL11.glPushMatrix();
		GL11.glOrtho((float)(-screenSize.getX()/2*c.getZ()), (float)(screenSize.getX()/2*c.getZ()), (float)(-screenSize.getY()/2*(float)c.getZ()), (float)(screenSize.getY()/2*c.getZ()), -1, 1);
	}
	public void revertPersp(){
		//Intended to be used to undo the perspective established by c
		GL11.glMatrixMode(GL11.GL_PROJECTION);
		GL11.glPopMatrix();
	}
	public void setCamLoc(Coordinate c){
		//Camera locations are defined in terms of game space, NEVER screen space!
		boundCam.setLocation(c);
	}
	public void bindCam(glECamera cam){
		boundCam = cam;
	}
	public void bindCam(int camID){
		if(camID >= 0 && camID < cameras.size()+1){
			if(camID == 0){
				boundCam = primaryCam;
			}else{
				boundCam = cameras.get(camID-1);
			}
		}
	}
	public glECamera getCam(){
		return boundCam;
	}
	public void draw(Coordinate c){
		//primaryCam.setLocation(c);
		numSpritesDrawnThisFrame = 0;
		GL11.glEnableClientState(GL11.GL_VERTEX_ARRAY);
		GL11.glEnableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
		//GL11.glDisable(GL11.GL_DEPTH_TEST);
		GL11.glEnable(GL11.GL_TEXTURE_2D);
		GL11.glEnable (GL11.GL_BLEND);
		GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);

		bindCam(CAM_PRIMARY);
		boundCam.setLocation(c);

		setMode(this.game);
		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glPushMatrix();
		GL11.glLoadIdentity();
		GL11.glTranslatef(-(float)c.getX(), -(float)c.getY(), 0);

		//System.out.println(effects.size());
		if(drawEffects){
			for(int i=0;i<effects.size();i++){
				if(effects.get(i).isVisible(boundCam)){
					//System.out.println("Drawing "+effects.get(i));
					drawEffect(effects.get(i));
					bindCam(CAM_PRIMARY);
				}
			}
		}
		if(!texturesPrimed){
			primeForTextures();
		}
		if(drawObjects){
			for(int i=0;i<objects.size();i++){
			//	if(visible(objects.get(i), c)){
					drawSprite(objects.get(i), (float)c.getZ());
			//	}
			}
		}
		if(shadows){
			for(int i=0;i<objects.size();i++){
				if(objects.get(i).shadable()){
					for(int j=0;j<lights.size();j++){
						stdio.debug(this, "Shadows requested, but code is not present. Write this please.");
						//cast a shadow from the light source to this object and past.
						//Make the distance the shadow is cast a function of brightness of the source/width of shadow
						//so most shadows would go maybe 400 units past, but large objects could cast 700-900 unit long shadows etc
					}
				}
			}
		}
		//drawLine(new Coordinate(0,0), getSize(), new Color(125,62,74));


		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glPopMatrix();
		revertPersp();
		GL11.glDisableClientState(GL11.GL_VERTEX_ARRAY);
		GL11.glDisableClientState(GL11.GL_TEXTURE_COORD_ARRAY);
		try{
		//	GL11.glFlush();
		}catch(NullPointerException npe){
			npe.printStackTrace();
		}
	//	System.out.println("Engine.draw] Could not complete; interrupted!");
	}
	public void update(){
		Display.update();
	}
	public void clear(){
		GL11.glClearColor(0f,0f,0f,1f);
		GL11.glClear(GL11.GL_COLOR_BUFFER_BIT);
	}
	public void drawSprite(glESprite s, float cameraDist){
		drawSprite(s, s.x, s.y, s.spin, cameraDist);
	}
	public void drawSprite(glESprite s, float x, float y, float spin, float cameraDist){

		boolean transformDone = false;

		String typeName = s.getTypeName();
		Integer currTex;
		int width;
		int height;
	//	int lookupTimeTotal = 0;
	//	boolean show = false;
	//	long temp;
	//	boolean tp = false;
	//	long totalTime = System.nanoTime();

		for(int i=0;i<objectDrawOrder.length;i++){
	//		temp = System.nanoTime();
	//		show = show | s.isVisible(objectDrawOrder[i]);
	//		lookupTimeTotal += System.nanoTime()-temp;
			if(s.isVisible(objectDrawOrder[i])){
				currTex = (Integer)loadedResources.get(typeName+objectDrawOrder[i]);
				if(currTex != null){
//
	//				temp = System.nanoTime();
	//				tp = tp | new glEImage(currTex, new Coordinate(x, y), s.getSize(objectDrawOrder[i])).isVisible(primaryCam);
	//				lookupTimeTotal += System.nanoTime()-temp;

					if(new glEImage(currTex, new Coordinate(x, y), s.getSize(objectDrawOrder[i])).isVisible(primaryCam)){
						//temp = System.nanoTime();
						if(!transformDone){
							GL11.glMatrixMode(GL11.GL_MODELVIEW);
							GL11.glPushMatrix();
							//GL11.glColor3f(1f,1f,1f);
							GL11.glTranslatef(x, y, 0f);
							GL11.glRotatef((float)(Math.toDegrees(spin)), 0, 0, 1);
							transformDone = true;
						}
	//					temp = System.nanoTime();
						float[] drawInfo = s.getDrawInfo(objectDrawOrder[i]);
	//					lookupTimeTotal += System.nanoTime()-temp;

						float alpha = drawInfo[2];
						if(cameraDist>drawInfo[0]){
							alpha += (cameraDist-drawInfo[0])*drawInfo[1];
						}
						alpha = Math.min(alpha, 1f);
						alpha = Math.max(alpha, 0f);
						//System.out.println(alpha);
	//					totalTime += System.nanoTime()-temp;
						if(alpha > 0.01){
							float scale = drawInfo[5];
							boolean revertScale = false;
							if(Float.isNaN(drawInfo[4])){
								revertScale = true;
							}else{
								if(cameraDist>drawInfo[3]){
									scale += (cameraDist-drawInfo[3])*drawInfo[4];
								}
							}
							if(revertScale == true){
								scale*=cameraDist;
							}
	//						temp = System.nanoTime();
							width = s.getImage(objectDrawOrder[i]).getWidth(null);
							height = s.getImage(objectDrawOrder[i]).getHeight(null);
	//						lookupTimeTotal += System.nanoTime()-temp;
							//width = Math.max(width, height);
							//height = width;
							width*=scale;
							height*=scale;

							width*=drawInfo[6];
							height*=drawInfo[6];
							Color spriteCol = s.getColor(objectDrawOrder[i]);
							//Color spriteCol = new Color(255, 255, 255, 255);
							//System.out.println(objectDrawOrder[i]+", " + spriteCol.getRed() +", "+spriteCol.getBlue()+", "+spriteCol.getGreen()+", "+alpha);
							//setColor(new Color(1f,1f,1f,1f));
							setColor(new Color(spriteCol.getRed()/255f, spriteCol.getBlue()/255f, spriteCol.getGreen()/255f, alpha));
							//setColor(new Colo



							drawImage(currTex, width, height);


						}
					}
				}
			}
		}
		if(transformDone){
			GL11.glPopMatrix();
		}
	//	if(tp && show){
		//	System.out.print(typeName+" Lookup time total: "+lookupTimeTotal);
		//	System.out.println(" Total time: "+(System.nanoTime()-totalTime));
	//	}
	}
	public void drawImage(glEImage image){
		if(image.isVisible(primaryCam)){
			drawImage(image.getAddress(), (float)image.getSize().getX(), (float)image.getSize().getY(), (float)image.getCenter().getX(), (float)image.getCenter().getY(), 0);
		}
	}
	public void drawImage(glEImage image, Coordinate loc){
		//System.out.println("Drawing image at "+loc+"; "+image.getSize());
		drawImage(image.getAddress(), (float)image.getSize().getX(), (float)image.getSize().getY(), (float)(image.getCenter().getX()+loc.getX()), (float)(image.getCenter().getY()+loc.getY()), 0);
	}
	public void drawImage(glEImage image, Coordinate loc, double r){
		//System.out.println("Drawing image at "+loc+"; "+image.getSize());
		drawImage(image.getAddress(), (float)image.getSize().getX(), (float)image.getSize().getY(), (float)(image.getCenter().getX()+loc.getX()), (float)(image.getCenter().getY()+loc.getY()), (float)r);
	}
	public void drawImage(glEImage image, Coordinate loc, Color c){
		//System.out.println("Drawing image at "+loc+"; "+image.getSize());
		drawImage(image.getAddress(), (float)image.getSize().getX(), (float)image.getSize().getY(), (float)(image.getCenter().getX()+loc.getX()), (float)(image.getCenter().getY()+loc.getY()), 0, c);
	}
	public void drawImage(int sprite, float width, float height){
		GL11.glEnable(GL11.GL_TEXTURE_2D);
		glETexture tex = textureList.get(sprite);
		bindTex(tex.getAddr());
		/*
		System.out.println("Drawing, size:" +width +", "+height);
		float[] coords = {-width/2, -height/2, -width/2, height/2, width/2, height/2, width/2, -height/2};
		float[] texs = {0, tex.texSpaceHeight(), 0, 0, tex.texSpaceWidth(), 0, tex.texSpaceWidth(), tex.texSpaceHeight()};
		FloatBuffer vertexBuf = BufferUtils.createFloatBuffer(coords.length);
		vertexBuf.put(coords);
		vertexBuf.flip();
		FloatBuffer texBuf = BufferUtils.createFloatBuffer(texs.length);
		texBuf.put(texs);
		texBuf.flip();

		GL11.glVertexPointer(2, 0, vertexBuf);
		GL11.glTexCoordPointer(2, 0, texBuf);
		GL11.glDrawArrays(GL11.GL_TRIANGLE_FAN, 0, coords.length/2);
		*/
		numSpritesDrawnThisFrame++;

		GL11.glBegin(GL11.GL_TRIANGLE_FAN);
		GL11.glTexCoord2f(0, tex.texSpaceHeight());
		GL11.glVertex2f(-width/2, -height/2);
		GL11.glTexCoord2f(0, 0);
		GL11.glVertex2f(-width/2, height/2);
		GL11.glTexCoord2f(tex.texSpaceWidth(), 0);
		GL11.glVertex2f(width/2, height/2);
		GL11.glTexCoord2f(tex.texSpaceWidth(), tex.texSpaceHeight());
		GL11.glVertex2f(width/2, -height/2);
		GL11.glEnd();

	}
	public void drawImage(int sprite, float width, float height, float x, float y, float spin){
		drawImage(sprite, width, height, x, y, spin, new Color(255, 255, 255));
	}
	public void drawImage(int sprite, float width, float height, float x, float y, float spin, Color c){
		glEImage tempImage = new glEImage(textureList.get(sprite).getAddr(), new Coordinate(x, y), new Coordinate(width, height));
		if(tempImage.isVisible(boundCam)){
			if(!texturesPrimed){
				primeForTextures();
			}
			GL11.glMatrixMode(GL11.GL_MODELVIEW);
			GL11.glPushMatrix();
			setColor(c);
			GL11.glTranslatef(x, y, 0f);
			GL11.glRotatef((float)(spin*360/Math.PI), 0, 0, 1);
			drawImage(sprite, width, height);
			GL11.glPopMatrix();
		}
	}
	public void drawEffect(glEEffect e){
		e.draw(this);
	}
	public void drawRegisteredEffects(){
		for(int i=0;i<effects.size();i++){
			drawEffect(effects.get(i));
		}
	}
	public void registerSprite(glESprite s){
		objects.add(s);
		if(!loaded(s)){
			load(s, s.getResources());
		}else{
			Integer[] indices = new Integer[s.numResources()];
			for(int i=0;i<indices.length;i++){
				indices[i] = (Integer)loadedResources.get(s.getTypeName().concat(s.getResources()[i]));
			}
			s.setSpriteIndices(indices);
		}
		for(int i=0;i<s.numResources();i++){
			setFilter(s.getSpriteIndices()[i], spriteFilter);
		}
	}
	public void registerEffect(glEEffect e){
		effects.add(e);
		if(!loaded(e)){
			load(e, e.getResources());
		}
	}
	public void deregisterSprite(glESprite s){
		objects.remove(s);
	}
	public void deregisterEffect(glEEffect e){
		effects.remove(e);
	}
	public static int nearPow2(int input){
		int returnable = 1;
		while(input>returnable){
			returnable = returnable*2;
		}
		return returnable;
	}
	public boolean loaded(glESprite resourceCollection){
		boolean returnable = true;
		String[] resourcesToTest = resourceCollection.getResources();
		String typeName = resourceCollection.getTypeName();
		for(int i=0;i<resourcesToTest.length;i++){
			if(loadedResources.get(typeName+resourcesToTest[i])==null){
				returnable = false;
			}
		}
		return returnable;
	}
	public boolean loaded(glEEffect effect){
		boolean returnable = true;
		String[] resourcesToTest = effect.getResources();
		for(int i=0;i<resourcesToTest.length;i++){
			if(loadedResources.get(resourcesToTest[i]) == null){
				returnable = false;
			}
		}
		return returnable;
	}
	public boolean loaded(glEAnimArchive archive){
		String name = archive.name();
		return loadedAnimations.get(name) != null;
	}
	public void drawLine(Coordinate start, Coordinate end, Color col){
		drawLine(start, end, col, col);
	}
	public void drawLine(Coordinate start, Coordinate end, Color startCol, Color endCol){
		drawLine(start, end, startCol, endCol, 2f);
	}
	public void drawLine(Coordinate start, Coordinate end, Color startCol, Color endCol, float width){
		float length = (float)Math.sqrt(Math.pow(end.getX()-start.getX(), 2) + Math.pow(end.getY()-start.getY(), 2));
		double angle = Math.atan2(end.getY()-start.getY(), end.getX()-start.getX());
		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glPushMatrix();
		GL11.glTranslatef((float)start.getX(), (float)start.getY(), 0f);
		GL11.glRotatef((float)Math.toDegrees(angle), 0, 0, 1);
		GL11.glDisable(GL11.GL_TEXTURE_2D);
		texturesPrimed = false;
		float[] vertexData = {0, -width/2, 0, width/2, length, width/2, length, -width/2};
		//float[] colorData =

		GL11.glBegin(GL11.GL_QUADS);

		setColor(startCol);
		GL11.glVertex2f(0, -width/2);
		GL11.glVertex2f(0, width/2);
		setColor(endCol);
		GL11.glVertex2f(length, width/2);
		GL11.glVertex2f(length, -width/2);

		GL11.glEnd();

		GL11.glPopMatrix();
		GL11.glEnable(GL11.GL_TEXTURE_2D);
	}
	public void drawCircle(Coordinate center, float radius, int degreesPerSegment, Color color){
		float x = (float)center.getX();
		float y = (float)center.getY();
		setColor(color);
		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glPushMatrix();
		GL11.glTranslatef(x, y, 0f);
		GL11.glDisable(GL11.GL_TEXTURE_2D);
		texturesPrimed = false;


		GL11.glBegin(GL11.GL_TRIANGLE_FAN);

		GL11.glVertex2f(0, 0);

		for(int i=0;i<360;i+=degreesPerSegment){
			GL11.glVertex2f((float)Math.cos(Math.toRadians(i))*radius, (float)Math.sin(Math.toRadians(i))*radius);
		}
		GL11.glVertex2f(radius, 0);
		GL11.glEnd();

		GL11.glPopMatrix();
	}
	public void fillRect(Coordinate topLeft, Coordinate bottomRight, Color c){
		double[] points = new double[8];
		points[0] = topLeft.getX();
		points[1] = topLeft.getY();
		points[2] = bottomRight.getX();
		points[3] = topLeft.getY();
		points[4] = bottomRight.getX();
		points[5] = bottomRight.getY();
		points[6] = topLeft.getX();
		points[7] = bottomRight.getY();
		fillPoly(points, c);
	}
	public void fillPoly(double[] points, Color c){
		GL11.glDisable(GL11.GL_TEXTURE_2D);
		setColor(c);
		GL11.glBegin(GL11.GL_POLYGON);
		for(int i=0;i<points.length/2;i++){
			GL11.glVertex2f((float)points[i*2], (float)points[i*2+1]);
		}
		GL11.glEnd();
		GL11.glEnable(GL11.GL_TEXTURE_2D);
		setColor(new Color(1f,1f,1f,1f));
		//do stuff
	}
	public void drawPoly(double[] points, Color c){
		GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_LINE);
		fillPoly(points, c);
		GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_FILL);
		//do stuff
	}
	public void drawPoly(double[] points, double rot, double x, double y, Color c){
		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glPushMatrix();
		GL11.glTranslatef((float)x, (float)y, 0);
		GL11.glRotatef((float)Math.toDegrees(rot), 0, 0 , 1);
		drawPoly(points, c);
		GL11.glPopMatrix();
	}
	public void fillPoly(double[] points, double rot, double x, double y, Color c){
		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glPushMatrix();
		GL11.glTranslatef((float)x, (float)y, 0);
		GL11.glRotatef((float)Math.toDegrees(rot), 0, 0, 1);
		fillPoly(points, c);
		GL11.glPopMatrix();
	}
	public void drawRect(Coordinate topLeft, Coordinate bottomRight, Color c){
		texturesPrimed = false;
		double[] points = new double[8];
		points[0] = topLeft.getX();
		points[1] = topLeft.getY();
		points[2] = bottomRight.getX();
		points[3] = topLeft.getY();
		points[4] = bottomRight.getX();
		points[5] = bottomRight.getY();
		points[6] = topLeft.getX();
		points[7] = bottomRight.getY();
		drawPoly(points, c);
	}
	public void primeForTextures(){
		GL11.glEnable(GL11.GL_TEXTURE_2D);
		GL11.glEnable (GL11.GL_BLEND);
		GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
		GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_FILL);
		texturesPrimed = true;
	}
	public void drawString(String s, double x, double y){
		drawString(s,x,y, new Color(255, 255, 255));
	}
	public void drawString(String s, double x, double y, Color c){
		setColor(c);
		//System.out.println(s + " " + x);
		//Remove the extraneous openGL calls at some point.
		//Provide a more standard method to prime the engine/OpenGL for
		//rendering as we need in another method.
		//Perhaps something such as primeTexturedRender()
		//	which would set a flag in opengl as primed, and any
		//	call to alter that would change the primed flag to false?
		//and for things such as fill() we would then call
		//	dePrime() which would prepare us for drawing in the expected
		//	space with the expected methodolgy, and of course set
		//	the primed flag to false.
		float internalX = (float)x;
		float internalY = (float)y;

		if(!texturesPrimed){
		//	primeForTextures();
		}
		char[] chars = s.toCharArray();
		if(loadedFont != null && chars.length > 0){

			GL11.glEnable(GL11.GL_TEXTURE_2D);
			float stringWidth = loadedFont.getWidth(s);
			float ratio = (float)loadedFont.getRatio();
			glETexture fontTex = textureList.get(loadedFont.getImageAddress());
			bindTex(fontTex.getAddr());
			float strHeight = (float)loadedFont.getHeight();

			GL11.glBegin(GL11.GL_QUADS);
			for(int i=0;i<chars.length;i++){
				int loc = loadedFont.charToLoc(chars[i]);

				GL11.glTexCoord2f(loc*ratio*fontTex.texSpaceWidth(), fontTex.texSpaceHeight());
				GL11.glVertex2f(internalX+(i)*8f-stringWidth/2, internalY-strHeight/2f);

				GL11.glTexCoord2f(loc*ratio*fontTex.texSpaceWidth(), 0);
				GL11.glVertex2f(internalX+(i)*8f-stringWidth/2, internalY-strHeight/2f+strHeight);

				GL11.glTexCoord2f((loc+8)*ratio*fontTex.texSpaceWidth(), 0);
				GL11.glVertex2f(internalX+(i+1)*8f-stringWidth/2, internalY-strHeight/2f+strHeight);

				GL11.glTexCoord2f((loc+8)*ratio*fontTex.texSpaceWidth(), fontTex.texSpaceHeight());
				GL11.glVertex2f(internalX+(i+1)*8f-stringWidth/2, internalY-strHeight/2f);
				numSpritesDrawnThisFrame++;
			}
			GL11.glEnd();
		}else{
	//		System.out.println("GLEngine.drawString]: Engine has no font loaded!");
		}
	}
	public void setColor(Color c){
		GL11.glColor4f((float)c.getRed()/255,(float)c.getGreen()/255,(float)c.getBlue()/255, (float)c.getAlpha()/255);
		//GL11.glColor4f(1f,1f,1f,c.getAlpha()/255f);
	}
	public void useShadows(boolean b){
		shadows = b;
	}
	public boolean usingShadows(){
		return shadows;
	}
	public void addLight(glELight l){
		lights.add(l);
	}
	public void removeLight(glELight l){
		lights.remove(l);
	}
	public FontDataAccelerated getFont(){
		return loadedFont;
	}
	public double getHeight(){
		return screenSize.getY();
	}
	public double getWidth(){
		return screenSize.getX();
	}
	public Coordinate getSize(){
		return screenSize;
	}
	public void bindTex(int texture){
		if(boundTex != texture){
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, texture);
			boundTex = texture;
		}
	}
	public void push(){
		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glPushMatrix();
	}
	public void pop(){
		GL11.glMatrixMode(GL11.GL_MODELVIEW);
		GL11.glPopMatrix();
	}
	public void translate(float x, float y){
		GL11.glTranslatef(x, y, 0.0f);
	}
	public void rotate(double rot){
	//	GL11.glRotated(rot,0,0,0);
	}
	public void mask(int type){
		if(type == ALLOW_STENCIL_ONLY){
			GL11.glColorMask(false, false, false, false);
			GL11.glDepthMask(false);
		}else if(type == ALLOW_COLOR_ONLY){
			GL11.glColorMask(true, true, true, true);
			GL11.glDepthMask(false);
		}else if(type == ALLOW_NONE){
			GL11.glColorMask(false, false, false, false);
			GL11.glDepthMask(false);
		}else if(type == ALLOW_ALL){
			GL11.glColorMask(true, true, true, true);
			GL11.glDepthMask(true);
		}
	}
	public void stencilMode(int mode, int value){
		if(mode == STENCIL_SET){
			GL11.glStencilFunc(GL11.GL_ALWAYS, value, 0xFFFFFFFF);
			GL11.glStencilOp(GL11.GL_REPLACE, GL11.GL_REPLACE, GL11.GL_REPLACE);
		}else if(mode == STENCIL_EQUAL){
			GL11.glStencilFunc(GL11.GL_EQUAL, value, 0xFFFFFFFF);
			GL11.glStencilOp(GL11.GL_KEEP, GL11.GL_KEEP, GL11.GL_KEEP);
		}
	}
	public void clearStencil(){
		GL11.glClearColor(0f, 0f, 0f, 1f);
		GL11.glClear(GL11.GL_STENCIL_BUFFER_BIT);
	}
	public void setDrawObjects(boolean b){
		drawObjects = b;
	}
	public void setDrawEffects(boolean b){
		drawEffects = b;
	}
	/*
	public int createPIPCamera(Coordinate loc, Coordinate scale, double rotation){
		int index = pipCameras.size();
		pipCameras.add(new glECamera(loc, scale, rotation));
		return index;
	}
	public void useCamera(int pipCam){
		currentCamera = pipCameras.get(pipCam);
	}
	*/
	public DisplayMode[] getModes(){
		return validModes;
	}
	public int getNumModes(){
		return validModes.length;
	}
	public int getCurrentMode(){
		return currDisplayMode;
	}
	public int getPreviousMode(){
		int returnable = currDisplayMode;
		returnable--;
		if(returnable<0){
			returnable = validModes.length-1;
		}
		currDisplayMode = returnable;
		return returnable;
	}
	public int getNextMode(){
		int returnable = currDisplayMode;
		returnable++;
		if(returnable>=validModes.length){
			returnable = 0;
		}
		currDisplayMode = returnable;
		return returnable;
	}
	public String getModeAsString(int mode){
		String returnable;
		DisplayMode toConvert = validModes[mode];
		returnable = Integer.toString(toConvert.getWidth()).concat("x").concat(Integer.toString(toConvert.getHeight())).concat("@"+Integer.toString(toConvert.getFrequency()));
		return returnable;
	}
	public int getError(){
		return GL11.glGetError();
	}
	public void setFilter(int image, int filterType){
		glETexture tex = textureList.get(image);
		switch(filterType){
			case FILTER_LINEAR:
				bindTex(tex.getAddr());
				GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_LINEAR);
				GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_LINEAR);
				break;
			case FILTER_NEAREST:
				bindTex(tex.getAddr());
				GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MIN_FILTER, GL11.GL_NEAREST);
				GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_MAG_FILTER, GL11.GL_NEAREST);
				break;
			default:
				break;
		}
	}
	public boolean closeRequested(){
		return Display.isCloseRequested();
	}

	public void drawCurve(Coordinate start, Coordinate end, Coordinate focus, Color startCol, Color endCol){
		//temporary curve code. Please implement splines when you get home. or see this. Please.
		Coordinate tempStart = start;
		Coordinate temp = focus.sub(start).mult(0.8);
		Coordinate tempEnd = tempStart.add(temp);
		drawLine(tempStart, tempEnd, startCol);
		tempStart = tempEnd;
		temp = focus.sub(end).mult(0.8);
		tempEnd = end.add(temp);
		drawLine(tempStart, tempEnd, startCol);
		tempStart = tempEnd;
		tempEnd = end;
		drawLine(tempStart, tempEnd, endCol);
	}

	public void fullscreenToDesktopResolution(){
		try{
			Display.setDisplayMode(Display.getDesktopDisplayMode());
			Display.setFullscreen(true);
			GL11.glViewport(0, 0, 1024, 600);
		}catch(LWJGLException e){
			e.printStackTrace();
		}
	}

	public void setTitle(String s){
		Display.setTitle(s);
	}
	public int getNumSpritesDrawn(){
		return numSpritesDrawnThisFrame;
	}
	public void setSpriteFilter(int filter){
		for(int i=0;i<objects.size();i++){
			for(int j=0;j<objects.get(i).numResources();j++){
				setFilter(objects.get(i).getSpriteIndices()[j], filter);
			}
		}
		spriteFilter = filter;
	}
	public void setAnimFilter(int filter){

	}
	public void setBackgroundFilter(int filter){

	}
/*	public void drawCurve(Coordinate start, Coordinate end, Coordinate centerpoint, int levelOfDetail, Color start, Color end){
		levelOfDetail = Math.max(levelOfDetail, 1);
		Coordinate deltaStart = start.sub(centerpoint);
		Coordinate deltaEnd = end.sub(centerpoint);
		double startAngle = Math.atan2(deltaStart.getY(), deltaStart.getX());
		double endAngle = Math.atan2(deltaEnd.getY(), deltaEnd.getX());
		double circumference = Physics.pythag(start.sub(centerpoint))*2*Math.PI;
		double curveLength = circumference*(endAngle-startAngle)/2*Math.PI;
		int numSegmentsRaw = (int)Math.round(curveLength)/levelOfDetail;
		for(int i=0;i<numSegmentsRaw;i++){
			for(int i=0;i<
*/
}
