/*
	If any of these methods are provided an output array of size smaller than expected,
	the low-value bytes are discarded. If an array of size larger than needed, it is
	filled from index 0.
*/

package com.ixee.dataops;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Random;

public class BitOperations{

	public static void main(String[] args){
		int numRounds = 1000000;
		int numTests = 10;
		Random numberGen = new Random(0);

		byte[] parts = new byte[4];
		double d = 0;
		System.out.println("Making first passes, calling some stuff a few times.");
		double startTime = System.nanoTime();
		for(int i=0;i<100000;i++){
			parts = BitOperations.toByteArray(numberGen.nextInt(10000));
			d = System.nanoTime();
		}
		System.out.println((d-startTime)/1000000 + "ms used for initialization");

		System.out.println("Starting tests of garbage-y methods.");
		for(int i=0;i<numTests;i++){
			System.out.println("Test "+i+"...");
			startTime = System.nanoTime();
			for(int j=0;j<numRounds;j++){
				parts = BitOperations.toByteArray(numberGen.nextInt(10000));
			}
			System.out.println("  time: "+(System.nanoTime()-startTime)/1000000+"ms");
		}
		System.out.println("\nStarting tests of 'clean' methods.");
		for(int i=0;i<numTests;i++){
			System.out.println("Test "+i+"...");
			startTime = System.nanoTime();
			for(int j=0;j<numRounds;j++){
				BitOperations.toByteArray(numberGen.nextInt(10000), parts);
			}
			System.out.println("  time: "+(System.nanoTime()-startTime)/1000000+"ms");
		}

	}
		//simply converts the input to an array of bytes and
		//calls swapEndianness() on it
	public static byte[] swapEndianness(int setOfBytes){
		return swapEndianness(toByteArray(setOfBytes));
	}


		//Swaps the endianness of the input array of bytes
		//takes byte[i], copies it to an internal byte,
		//moves byte[byte.length-i] to byte[i]
		//then copies the internal byte to byte[byte.length-1]

		//proceeds from i=0 to i=byte.size/2, then returns the resultant array.
	public static byte[] swapEndianness(byte[] toSwap){
		byte[] internalBuffer = toSwap;
		int size = toSwap.length;
		byte buffer;
		for(int i=0;i<size/2;i++){
			buffer = internalBuffer[i];
			internalBuffer[i] = internalBuffer[size-(i+1)];
			internalBuffer[size-(i+1)] = buffer;
		}
		return internalBuffer;
	}

		//Converts the eight bytes in a double to a byte array
	public static byte[] toByteArray(double d){
		byte[] returnable;

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(bos);

		try{
			dos.writeDouble(d);
		}catch(IOException ioe){}

		returnable = bos.toByteArray();

		return returnable;
	}
		//Converts the four bytes in a float to a byte array
	public static byte[] toByteArray(float d){
		byte[] returnable;

		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(bos);

		try{
			dos.writeFloat(d);
		}catch(IOException ioe){}

		returnable = bos.toByteArray();

		return returnable;
	}
		//Converts the two bytes in a short to a byte array
	public static byte[] toByteArray(short s){
		byte[] returnable = new byte[2];
		toByteArray(s, returnable);
		return returnable;
	}

		//Converts the four bytes in an int to a byte array
	public static byte[] toByteArray(int i){
		byte[] returnable = new byte[4];
		toByteArray(i, returnable);
		return returnable;
	}

	public static byte[] toByteArray(long l){
		byte[] returnable = new byte[8];
		toByteArray(l, returnable);
		return returnable;
	}
	public static void toByteArray(long l, byte[] b){
		toByteArray(l, b, 0);
	}
	public static void toByteArray(long l, byte[] b, int offset){
		long mask = 0xFFL;
		int max = Math.min(b.length-offset, 8);
		for(int i=0;i<max;i++){
			b[max-i-1+offset] = (byte)((l & mask)>>(i*8));
			mask = mask << 8;
		}
	}

	public static long toLong(byte[] b){
		long returnable = 0x0000000000000000L;

		returnable = ((long)(b[0]&0xff))<<56|((long)(b[1]&0xff))<<48|((long)(b[2]&0xff))<<40|((long)(b[3]&0xff))<<32|((long)(b[4]&0xff))<<24|((long)(b[5]&0xff))<<16|((long)(b[6]&0xff))<<8|((long)(b[7]&0xff));

		return returnable;
	}


		//Converts an array (hopefully, of four bytes) to an int
	public static int toInt(byte[] b){
		int returnable = 0x00000000;

		returnable = (b[0]&0xff)<<24|(b[1]&0xff)<<16|(b[2]&0xff)<<8|(b[3]&0xff);

		return returnable;
	}

		//Converts an array (hopefully, of two bytes) to a short
	public static short toShort(byte[] b){
		short returnable = 0x0000;

		returnable = (short)(((b[0]<<8)&0xFF00)|(b[1]&0xFF));

		return returnable;
	}
		//Converts an array (hopefully of eight bytes) to a double
	public static double toDouble(byte[] b){
		double returnable = Double.NaN;

		ByteArrayInputStream bis = new ByteArrayInputStream(b);
		DataInputStream dis = new DataInputStream(bis);

		try{
			returnable = dis.readDouble();
		}catch(IOException ioe){}

		return returnable;
	}
		//Converts an array (hopefully of eight bytes) to a double
	public static float toFloat(byte[] b){
		float returnable = Float.NaN;

		ByteArrayInputStream bis = new ByteArrayInputStream(b);
		DataInputStream dis = new DataInputStream(bis);

		try{
			returnable = dis.readFloat();
		}catch(IOException ioe){}

		return returnable;
	}
	public static boolean[] toBooleanBits(byte[] b){
		boolean[] returnable = new boolean[b.length*8];
		byte[] toWorkOn = b;
		for(int i=0;i<b.length;i++){
			for(int inc=0;inc<8;inc++){
				if((0x01 & toWorkOn[i]) == 1){
					returnable[inc+i*8] = true;
				}else{
					returnable[inc+i*8] = false;
				}
				toWorkOn[i] = (byte)(toWorkOn[i]>>>0x01);
			}
		}
		return returnable;
	}
	public static byte setBit(byte b, int index){
		byte returnable = b;
		byte mask=0x00;
		if(index<8 && index >= 0){
			mask = (byte)(0x01<<index);
		}
		returnable = (byte)(returnable|mask);
		return returnable;
	}
	public static boolean hasBit(byte b, int index){
		int operable = b;
		int mask = 0x00;
		if(index<8 && index>=0){
			mask = 0x01<<index;
			operable = operable&mask;
			operable = operable>>>index;
			boolean returnable = false;
			if(operable == 1){
				returnable = true;
			}else{
				returnable = false;
			}
			return returnable;
		}else{
			System.out.println("BitOperations.132]: Index out of bounds. Put an exception here.");
			return false;
		}
	}
}