 /*
	TODO: Make the size of the color table value in the CompressedImageRect dependant on the size of the color table itself!
		Thus, when encoding it all, reserve ceil(log2(colorTable.size())/log2(256)) bytes for the color table offsets.
*/

/*
	All methods specifying some range of pixels is inclusive of the lower/upper bounds, non-inclusive of right/bottom bounds.
*/

/*
	Many of the function calls here are somewhat ambiguous!

	This is because most arguments used are static to the class, and don't need to be passed.

	Things such as the current working image and sub-frames to work in are never passed around.

	All the methods here work on the current "frame" of the image; the current rectangle we are testing as contiguous, filling, or compressing.

	The only thing that changes the values of the frame stack is compress() !
		This is because it is up to compress() to decide when we can consider the current frame compressed, or if we need to compress sub-rects.

*/
/*
	Format:
		int length
		int width
		byte colorTable
		int[] colors

		int numSquares

		struct compressedRect{
			byte color
			int topLeftPixelNumber
			int bottomRightPixelNumber
		}
		compressedRect[numSquares]

*/
package com.ixee.dataops;

import java.util.ArrayList;
import com.ixee.io.stdio;
import java.awt.image.BufferedImage;
import com.ixee.dataops.BitOperations;
import com.ixee.datastruct.Coordinate;
import java.util.LinkedList;
import java.util.*;
import java.awt.image.*;
import java.awt.*;

public class ImageCompression{
	static ArrayList<Integer> colorTable = new ArrayList<Integer>();
	static LinkedList<ImageCompressionFrame> frameStack = new LinkedList<ImageCompressionFrame>();
	static int[][] data;

	public static void uncompress(byte[] input){
		MarkedInputStream mis = new MarkedInputStream(input);
		byte[] buffer = mis.get(4);
		int width = BitOperations.toInt(mis.get(4));
		int height = BitOperations.toInt(mis.get(4));
		int size = BitOperations.toInt(mis.get(4));
		int numColors = BitOperations.toInt(mis.get(4));

		colorTable = new ArrayList<Integer>();

		for(int i=0;i<numColors;i++){
			colorTable.add(new Integer(BitOperations.toInt(mis.get(4))));
		}
		while(mis.remaining()>9 && mis.remaining()%9 == 0){
			frameStack.push(Imag
		}

	public static byte[] compress(BufferedImage img){
		frameStack.clear();
		colorTable.clear();
		data = new int[img.getWidth(null)][img.getHeight(null)];
		colorTable.add(new Integer(0xFFFFFFFF));
		for(int i=0;i<data.length;i++){
			for(int j=0;j<data[0].length;j++){
				data[i][j] = img.getRGB(i, j);
			}
		}
		LinkedList<ImageCompressionRect> rects = new LinkedList<ImageCompressionRect>();;
		if(data.length != 0 && data[0].length != 0){
			colorTable.add(new Integer(data[0][0]));
			rects = compress(0, 0, data.length-1, data[0].length-1, 0);
		}else{
			stdio.debugAndLog("Cannot compress image with dimension of 0");
		}

		byte[] returnable = new byte[4+4+4+1*4+colorTable.size()*4+rects.size()*(2*4+1)];
		//width, height, Total output size, color table size, color table entries (a list of ints), compressed rects in the form of (topLeftInt, bottomRightInt, byteColor);
		int offset = 0;

		//System.out.println("Color table size: "+colorTable.size());

		stdio.log("Image dimensions: "+data.length+", "+data[0].length);

		byte[] currInt = BitOperations.toByteArray(data.length);
		System.arraycopy(currInt, 0, returnable, offset, 4);
		offset += 4;
		BitOperations.toByteArray(data[0].length, currInt);
		System.arraycopy(currInt, 0, returnable, offset, 4);
		offset += 4;
		BitOperations.toByteArray(colorTable.size(), currInt);
		System.arraycopy(currInt, 0, returnable, offset, 4);
		offset += 4;
		for(int i=0;i<colorTable.size();i++){
		//	System.out.println("Storing color table entry "+i+", "+colorTable.get(i));
			BitOperations.toByteArray(colorTable.get(i), currInt);	//Break the current color into bytes
			System.arraycopy(currInt, 0, returnable, offset, 4);	//currInt should never be more than four bytes.
			offset += 4;					//One int. No more, no less.
		}
		stdio.log("Color table written, "+(offset)+" bytes used so far, 4 for total size, "+colorTable.size()+" colors, each 4 bytes.");
		ListIterator<ImageCompressionRect> rectIterator = rects.listIterator();
		byte[] currRectData = new byte[9];
		while(rectIterator.hasNext()){
			ImageCompressionRect currRect = rectIterator.next();
			currRect.toBytes(currRectData);
			System.arraycopy(currRectData, 0, returnable, offset, 9);
			offset+=9;
		}
		return returnable;
	}
		//Filling the imageData array with all the pixels
	//

	private static LinkedList<ImageCompressionRect> compress(int leftBound, int upperBound, int rightBound, int bottomBound, int preferredColInd){
		//So the first thing we do is set up a frame to compress inside and push it onto the stack.

		frameStack.push(new ImageCompressionFrame(leftBound, upperBound, rightBound, bottomBound));

		//The parent frame is the absolute word of god. Outside this frame the image DOES NOT EXIST.

		ImageCompressionFrame parentFrame = frameStack.peekFirst();

		int preferredColIndex = preferredColInd;
		//System.out.println("Compressing rect; left: "+parentFrame.left()+" to right: "+parentFrame.right()+" :: upper: "+parentFrame.top()+" to bottom: "+parentFrame.bottom());

		int height = parentFrame.bottom() - parentFrame.top();
		int width = parentFrame.right() - parentFrame.left();

		LinkedList<ImageCompressionRect> returnable = new LinkedList<ImageCompressionRect>();

		//System.out.println("Pass depth "+frameStack.size());
		//System.out.println(frameStack.peekFirst());

		while(!contiguous()){
		//	System.out.println("Doing a round of why-is-this-not-contiguous!");
			//find the disruptible subregion, size it, and compress it.
			int subUpperBound, subLefterBound;
			subUpperBound = -1;
			subLefterBound = -1;
			int subRighterBound, subBottomBound;
			subRighterBound = -1;
			subBottomBound = -1;
			boolean boundCornerFound = false;
			int scanXDist = 0;
			int scanYDist = 0;
			int startCol = data[leftBound][upperBound];
			int cornerCol = -1;
			if(colorTable.indexOf(startCol) == -1){
				colorTable.add(new Integer(startCol));
			}
			while(!boundCornerFound){
			//System.out.println("Compressing! Non-contiguous region found.");
				scanXDist = Math.min(parentFrame.right(), scanXDist+parentFrame.left())-parentFrame.left();
				scanYDist = Math.min(parentFrame.bottom(), scanYDist+parentFrame.top())-parentFrame.top();
				//System.out.println("UpperLeft scan settings: "+scanXDist+", "+scanYDist);

				int xIter = 0;
				int yIter = 0;
				for(int i=0;i<=Math.max(scanXDist, scanYDist) && !boundCornerFound;i++){
					int colToTest = -1;
					if(!boundCornerFound){
						colToTest = data[parentFrame.left()+scanXDist][parentFrame.top()+yIter];
						//System.out.println("Orig: "+startCol +", test: "+colToTest);
						boundCornerFound = (colToTest != startCol);

						if(colorTable.indexOf(colToTest) == -1){
							colorTable.add(new Integer(colToTest));
						}

						if(boundCornerFound){
							cornerCol = colToTest;
							subLefterBound = parentFrame.left()+scanXDist;
							subUpperBound = parentFrame.top()+yIter;
			//				System.out.println("UpperLeft found at "+subLefterBound+", "+subUpperBound+"!");

						}

					}
					if(!boundCornerFound){
						colToTest = data[parentFrame.left()+xIter][parentFrame.top()+scanYDist];
						//System.out.println("Orig: "+startCol +", test: "+colToTest);
						boundCornerFound = (colToTest != startCol);

						if(colorTable.indexOf(colToTest) == -1){
							colorTable.add(new Integer(colToTest));
						}


						if(boundCornerFound){
							cornerCol = colToTest;
							subLefterBound = parentFrame.left()+xIter;
							subUpperBound = parentFrame.top()+scanYDist;

			//				System.out.println("UpperLeft found at "+subLefterBound+", "+subUpperBound+"!");
						}
					}
					xIter++;
					yIter++;
					xIter = Math.min(scanXDist, xIter);
					yIter = Math.min(scanYDist, yIter);
				}
				scanXDist++;
				scanYDist++;
			}
			//System.out.println("UpperLeft found at "+subLefterBound+", "+subUpperBound+"!");
			//System.out.println("Color: "+cornerCol);
			boundCornerFound = false;
			scanXDist = 0;
			scanYDist = 0;
			while(!boundCornerFound){
				//System.out.println("Bounds: "+
				scanXDist = Math.max(subLefterBound, Math.max(parentFrame.left(), parentFrame.right()+scanXDist))-parentFrame.right();
				scanYDist = Math.max(subUpperBound, Math.max(parentFrame.top(), parentFrame.bottom()+scanYDist))-parentFrame.bottom();
				//System.out.println("BottomRight scan settings: "+scanXDist+", "+scanYDist);
				int xIter = 0;
				int yIter = 0;
				for(int i=0;i>=Math.min(scanXDist, scanYDist) && !boundCornerFound;i--){
					int colToTest = -1;
					if(!boundCornerFound){
						//System.out.println("scanXDist: "+scanXDist+", scanYDist: "+scanYDist);
			//			System.out.println("Testing at "+(rightBound+scanXDist)+", "+(bottomBound+yIter));
						colToTest = data[parentFrame.right()+scanXDist][parentFrame.bottom()+yIter];

						boundCornerFound = ((colToTest != startCol) && (colToTest == cornerCol));

						if(colorTable.indexOf(colToTest) == -1){
							colorTable.add(new Integer(colToTest));
						}

						if(boundCornerFound){
							//do some test to see if boundCorner is connected to the first corner. !!!!
							//Also once we set that up do some extra logic to skip tiles of this color until the color changes.

							//Nothing too much.

							subRighterBound = parentFrame.right()+scanXDist;
							subBottomBound = parentFrame.bottom()+yIter;
						}
					}
					if(!boundCornerFound){
			//			System.out.println("Testing at "+(rightBound+xIter)+", "+(bottomBound+scanYDist));
						colToTest = data[parentFrame.right()+xIter][parentFrame.bottom()+scanYDist];

						boundCornerFound = ((colToTest != startCol) && (colToTest == cornerCol));

						if(colorTable.indexOf(colToTest) == -1){
							colorTable.add(new Integer(colToTest));
						}

						if(boundCornerFound){
							//do some test to see if boundCorner is connected to the first corner. !!!!
							subRighterBound = parentFrame.right()+xIter;
							subBottomBound = parentFrame.bottom()+scanYDist;
						}
					}

					if(boundCornerFound){
						frameStack.push(new ImageCompressionFrame(subLefterBound, subUpperBound, subRighterBound, subBottomBound));
						boolean connected = connected(); //Tests to see if the top left of the current frame is connected to the bottom right of the current frame.
						frameStack.pop();
						if(!connected){
							boundCornerFound = false;
						}
					}
					xIter--;
					yIter--;
					xIter = Math.max(scanXDist, xIter);
					yIter = Math.max(scanYDist, yIter);
				}
				scanXDist--;
				scanYDist--;
			}
			//System.out.println("BottomRight found at "+subRighterBound+", "+subBottomBound+"!");
			boundCornerFound = false;
			scanXDist = 0;
			scanYDist = 0;


			returnable.addAll(compress(subLefterBound, subUpperBound, subRighterBound, subBottomBound, colorTable.indexOf(startCol)));

			ImageCompressionFrame compressedFrame = new ImageCompressionFrame(subLefterBound, subUpperBound, subRighterBound, subBottomBound);

			frameStack.push(compressedFrame);

			//System.out.println("Counting perimeter pixels to pick border color.");

			int[] colorCount = new int[colorTable.size()];
			//Going along the bottom edge
			for(int i=compressedFrame.left();i<=compressedFrame.right();i++){
				if(i>=parentFrame.left() && i<=parentFrame.right() && compressedFrame.bottom()+1 <= parentFrame.bottom()){
					int colorToCompareTo = data[i][compressedFrame.bottom()+1];
			//		System.out.println("Color: "+colorToCompareTo+" Corner: "+cornerCol);
			//		if(colorToCompareTo != cornerCol){
						colorCount[colorTable.indexOf(colorToCompareTo)]++;
			//		}
				}
			}
			//Going along the top edge
			//System.out.println("That one done.");
			for(int i=compressedFrame.right();i>=compressedFrame.left();i--){
				if(i>=parentFrame.left() && i <= parentFrame.right() && compressedFrame.top()-1 >= parentFrame.top()){
					int colorToCompareTo = data[i][compressedFrame.top()-1];
			//		System.out.println("Color: "+colorToCompareTo+" Corner: "+cornerCol);
			//		if(colorToCompareTo != cornerCol){
						colorCount[colorTable.indexOf(colorToCompareTo)]++;
			//		}
				}
			}
			//Going along the left edge
			//System.out.println("That one done.");
			for(int i=compressedFrame.top();i<=compressedFrame.bottom();i++){
				if(i>=parentFrame.top() && i <= parentFrame.bottom() && compressedFrame.left()-1 >= parentFrame.left()){
					int colorToCompareTo = data[compressedFrame.left()-1][i];
			//		System.out.println("Color: "+colorToCompareTo+" Corner: "+cornerCol);
			//		if(colorToCompareTo != cornerCol){
						colorCount[colorTable.indexOf(colorToCompareTo)]++;
					}
			//	}
			}
			//Going along the right edge
			//System.out.println("That one done.");
			for(int i=compressedFrame.bottom();i>=compressedFrame.top();i--){
				if(i>=parentFrame.top() && i <= parentFrame.bottom() && compressedFrame.right()+1 <= parentFrame.right()){
					int colorToCompareTo = data[compressedFrame.right()+1][i];
			//		System.out.println("Color: "+colorToCompareTo+" Corner: "+cornerCol);
			//		if(colorToCompareTo != cornerCol){
						colorCount[colorTable.indexOf(colorToCompareTo)]++;
			//		}
				}
			}
			//System.out.println("That one done.");

			int colorIndexToReplace = -1;
			int max = colorCount[0];
			//for(int i=0;i<colorCount.length;i++){
			//	System.out.println("Frequency of color "+i+" ("+colorTable.get(i)+") is "+colorCount[i]);
			//	if(colorCount[i] >= max){
			//		colorIndexToReplace = i;
			//		max = colorCount[i];
			//	}
			//}

	//		if(colorIndexToReplace == -1 || (compressedFrame.right() == parentFrame.right() && compressedFrame.left() == parentFrame.left()) || (compressedFrame.top() == parentFrame.top() && compressedFrame.bottom() == compressedFrame.bottom())){
	//			System.out.println("OBJECTION");
	//			colorIndexToReplace = preferredColIndex;
	//		}

			colorIndexToReplace = colorTable.indexOf(data[parentFrame.left()][parentFrame.top()]);

		//	System.out.println("Color to use: "+colorIndexToReplace+" aka "+colorTable.get(colorIndexToReplace));

			fill(colorTable.get(colorIndexToReplace));

			frameStack.pop();
		}

		//if(contiguous(data, leftBound, upperBound, width, height){
		returnable.add(new ImageCompressionRect(
			(byte)colorTable.indexOf(data[leftBound][upperBound]),
			upperBound*data[0].length+leftBound,
			(bottomBound)*data[0].length+(rightBound)
		));
		//}else{
		frameStack.pop();
		System.out.println("Compressed into "+returnable.size()+" rects");

		return returnable;
	}
	private static void fill(int fillColor){
		/*
			Fills the current frame with fillColor
		*/

		ImageCompressionFrame currFrame = frameStack.peekFirst();

		//System.out.print("Filling rect with ");
		//stdio.print(BitOperations.toByteArray(fillColor));
		//System.out.println("; left: "+currFrame.left()+" to right: "+currFrame.right()+" :: upper: "+currFrame.top()+" to bottom: "+currFrame.bottom());
		//rewrite this to use system.arraycopy().

		for(int i=currFrame.left();i<=currFrame.right();i++){
			for(int j=currFrame.top();j<=currFrame.bottom();j++){
				//System.out.println("Filling "+i+", "+j+" with "+fillColor);
				data[i][j] = fillColor;
			}
		}
	}
	private static boolean contiguous(){
		/*
			No arguments, just check to see if the image defined by the top frame is contiguous.
		*/
		ImageCompressionFrame currFrame = frameStack.peekFirst();
	//	System.out.println("Testing for contiguous rect; left: "+leftBound+" to right: "+rightBound+" :: upper: "+upperBound+" to bottom: "+bottomBound);
	//	System.out.println(data);
		int color = data[currFrame.left()][currFrame.top()];
		boolean returnable = true;
		for(int i=currFrame.left();i<=currFrame.right() && returnable;i++){
			for(int j=currFrame.top();j<=currFrame.bottom() && returnable;j++){
				returnable = returnable & (data[i][j] == color);
			}
		}
		//System.out.println("Contiguous? "+returnable);
		return returnable;
	}
	private static boolean connected(){
		/*
			No arguments, check to see if the topLeft, bottomRight corners of the current frame are connected within
			the frame just above.
		*/
		ImageCompressionFrame currFrame = frameStack.pop();
		ImageCompressionFrame parentFrame = frameStack.peekFirst();
		frameStack.push(currFrame);

		int color = data[currFrame.left()][currFrame.top()];

		//Do crazy A* things to get this figured out.

		return true;
	}

}
class ImageCompressionFrame{
	Coordinate topLeft;
	Coordinate bottomRight;

	public ImageCompressionFrame(int left, int up, int right, int down){
		topLeft = new Coordinate(left, up);
		bottomRight = new Coordinate(right, down);
	}
	public ImageCompressionFrame(Coordinate tl, Coordinate br){
		topLeft = tl;
		bottomRight = br;
	}
	public int left(){
		return topLeft.intX();
	}
	public int right(){
		return bottomRight.intX();
	}
	public int top(){
		return topLeft.intY();
	}
	public int bottom(){
		return bottomRight.intY();
	}
	public String toString(){
		return "Frame: l: "+left()+", u: "+top()+", r: "+right()+", b: "+bottom();
	}
}
class ImageCompressionRect{
	byte color;
	int topLeft;
	int bottomRight;

	public ImageCompressionRect(byte col, int tl, int br){
		color = col;
		topLeft = tl;
		bottomRight = br;
	}
	public ImageCompressionRect(byte[] data){
		MarkedInputStream mis = new MarkedInputStream(data);
		color = mis.get();
		topLeft = BitOperations.toInt(mis.get(4));
		bottomRight = BitOperations.toInt(mis.get(4));
	}
	public void toBytes(byte[] dest){

		//REWRITE THIS.
		//Done.

		int offset = 0;
		dest[0] = color;
		offset++;

		BitOperations.toByteArray(topLeft, dest, 1);

		offset += 4;

		BitOperations.toByteArray(bottomRight, dest, 5);
	}
}