//defines a coordinate, angular position, velocity, angular velocity, or
//any other triplet of values.
//
//contains a Double[] where [0] is x, [1] is y, and [2] is z
//
//+x is to the left, +y is up, and +z is forward
//
//for rotations, x = yz plane, y = xz plane, and z = xy plane
// for x-rotation, 0 is +z axis
// for y-rotation, 0 is +z axis
// for z-rotation, 0 is +x axis

package com.ixee.datastruct;

public class Coordinate{
	private double[] poss = new double[3];

	public Coordinate(double x, double y){
		poss[0] = x;
		poss[1] = y;
		poss[2] = 0;
	}
	public Coordinate(double x, double y, double z){
		poss[0]=x;
		poss[1]=y;
		poss[2]=z;
	}
	public Coordinate(Coordinate c){
		poss[0]=c.getX();
		poss[1]=c.getY();
		poss[2]=c.getZ();
	}
	public Coordinate(double[] d) throws ArrayIsWrongSizeException{
		if(d.length == 3){
			poss[0] = d[0];
			poss[1] = d[1];
			poss[2] = d[2];
		}else{
			throw new ArrayIsWrongSizeException(3, d.length);
		}
	}
	public double getX(){
		return poss[0];
	}
	public double getY(){
		return poss[1];
	}
	public double getZ(){
		return poss[2];
	}
	public int intX(){
		return (int)Math.round(poss[0]);
	}
	public int intY(){
		return (int)Math.round(poss[1]);
	}
	public int intZ(){
		return (int)Math.round(poss[2]);
	}
	public double[] getCoords(){
		return poss;
	}
	public double getCoords(int i) throws IndexOutOfBoundsException{
		if(i<poss.length){
			return poss[i];
		}else{
			throw new IndexOutOfBoundsException();
		}
	}
	public int getInt(int i){
		int returnable = 0;
		try{
			returnable = (int)Math.round(getCoords(i));
		}catch(IndexOutOfBoundsException ioobe){
			ioobe.printStackTrace();
		}
		return returnable;
	}
	public void setX(double x){
		poss[0]=x;
	}
	public void setY(double y){
		poss[1]=y;
	}
	public void setZ(double z){
		poss[2]=z;
	}
	public void setCoords(double[] d) throws ArrayIsWrongSizeException{
		if(d.length!=3){
			throw new ArrayIsWrongSizeException(3, d.length);
		}else{
			for(int i=0;i<3;i++){
				poss[i]=d[i];
			}
		}
	}
	public void setCoords(int i, double d) throws IndexOutOfBoundsException{
		if(i>2){
			throw new IndexOutOfBoundsException();
		}else{
			poss[i]=d;
		}
	}
	public void setCoords(Coordinate c){
		for(int i=0;i<3;i++){
			poss[i]=c.getCoords(i);
		}
	}
	public boolean isCloseEnough(Coordinate c, double threshold){
		boolean closeEnough = true;
		for(int i=0;i<3;i++){
			if(Math.abs(this.getCoords(i)-c.getCoords(i))>threshold){
				closeEnough = false;
			}
		}
		return closeEnough;
	}
	public Coordinate add(Coordinate c){
		return new Coordinate(poss[0]+c.getX(), poss[1]+c.getY(), poss[2]+c.getZ());
	}
	public Coordinate sub(Coordinate c){
		return new Coordinate(poss[0]-c.getX(), poss[1]-c.getY(), poss[2]-c.getZ());
	}
	public Coordinate mult(double d){
		return new Coordinate(poss[0]*d, poss[1]*d, poss[2]*d);
	}
	public String toString(){
		String returnable = "";
		returnable = returnable.concat(getX()+" ");
		returnable = returnable.concat(getY()+" ");
		returnable = returnable.concat(getZ()+"");
		return returnable;
	}
}