/*
	This class defines a connection between a server and a client.

	It internally buffers ServCliPackets until Read/Write are called.

	This also runs as a thread on its own, so things done here should probably be ensured to be thread-safe.
		(MOST OF IT PROBABLY ISN'T.)
*/
/*
	Also, set up some ~low level~ protocol here.

	Generic connection status messages, heartbeat, and latency measuring should all be on this layer.
*/
package com.ixee.net;

import java.net.Socket;
import java.net.InetAddress;
import java.net.*;
import java.util.LinkedList;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import com.ixee.io.stdio;
import com.ixee.dataops.BitOperations;

public class ServCliConn extends Thread{
	Socket sock;
	String localAddress = "";
	String remoteAddress = "";
	volatile LinkedList<ServCliPacket> InputBuffer;
	volatile LinkedList<ServCliPacket> OutputBuffer;
	BufferedOutputStream output;
	BufferedInputStream input;

	ServCliPacket incomingPacket = null;

	InetSocketAddress lastValidDest;

	volatile boolean retry = true;
	int retryCount = 3;

	volatile boolean allowReconnect = true;

	String connectionStatus;

	int timeoutTime = 5000;

	volatile int cliID = -1;

	volatile private boolean close = false;

	volatile private boolean killed = false;

	public ServCliConn(){
		sock = new Socket();

		connectionStatus = "No connection attempt yet.";
		OutputBuffer = new LinkedList<ServCliPacket>();
		InputBuffer = new LinkedList<ServCliPacket>();
	}


	public ServCliConn(String address, int port){
		connectionStatus = "No connection attempt yet.";
		OutputBuffer = new LinkedList<ServCliPacket>();
		InputBuffer = new LinkedList<ServCliPacket>();
		try{
			sock = new Socket();
			connect(address, port);

			output = new BufferedOutputStream(sock.getOutputStream());
			input = new BufferedInputStream(sock.getInputStream());

		}catch(Exception e){
			connectionStatus = "Cannot establish connection.";
			e.printStackTrace();
			close = true;
		}
	}
	public ServCliConn(Socket so){
		connectionStatus = "No connection attempt yet.";
		OutputBuffer = new LinkedList<ServCliPacket>();
		InputBuffer = new LinkedList<ServCliPacket>();
		try{
			sock = so;
			output = new BufferedOutputStream(sock.getOutputStream());
			input = new BufferedInputStream(sock.getInputStream());
			remoteAddress = sock.getRemoteSocketAddress().toString();
			localAddress = sock.getLocalAddress().toString();
		}catch(Exception e){
			e.printStackTrace();
			close = true;
		}
	}
	public int getClientID(){
		return cliID;
	}
	public void setClientID(int i){
		cliID = i;
	}
	public String getIP(){
		return localAddress;
	}
	public String remoteIP(){
		String returnable = remoteAddress;
//		if(sock.isConnected()){
//			returnable = ;
//		}
		return returnable;
	}
	private synchronized void writeBuffer(){
		//System.out.println("Writing!");
		if(output != null){
			try{
				while(OutputBuffer.size() > 0){
		//			System.out.println("Writing packet");
					output.write(OutputBuffer.pollFirst().asBytes());
				}
				output.flush();
			}catch(IOException ioe){
				close();
			}
		}else{
			stdio.debugAndLog(this, "Connection not established. Warning! Discarding "+OutputBuffer.size()+" messages!");
		}
	//derp
	}
	private synchronized void read(){
		//System.out.println("Reading!");
		boolean reading = true;
		try{
			while(reading && sock.isConnected()){
			//	if(input.available() != 0){
			//		System.out.println("Available: "+input.available());
			//	}
			//	if(InputBuffer.size() > 1 || (InputBuffer.size() > 0 && InputBuffer.peekLast().isComplete())){
			//		System.out.println(InputBuffer.size()+" complete packets.");
			//	}
			//	System.out.print("Packets size: "+InputBuffer.size()+", last size is ");
			//	if(InputBuffer.peekLast() != null){
			//		System.out.println("complete: "+InputBuffer.peekLast().isComplete()+", remaining: "+InputBuffer.peekLast().getRemaining());
			//	}
				if((incomingPacket == null || incomingPacket.isComplete()) && input.available()>4){
			//		System.out.println("Reading new packet");
					byte[] header = new byte[4];
					byte typeByte = (byte)input.read();
 					input.read(header, 0, 4);
			//		System.out.println("Type: "+typeByte);
			//		System.out.println("Size0: "+size0);
			//		System.out.println("Size1: "+size1);
			//		System.out.println("Packet data size: "+BitOperations.toInt(header));
					incomingPacket = new ServCliPacket(typeByte, BitOperations.toInt(header));
			//		System.out.println(InputBuffer.get(InputBuffer.size()-1).toString());

				}else if(incomingPacket != null && !incomingPacket.isComplete()){
			//		System.out.println("Filling existing packet");
					byte[] buffer = new byte[Math.min(incomingPacket.getRemaining(), input.available())];
					input.read(buffer);
					incomingPacket.appendData(buffer);
			//		System.out.println("Read packet of type "+InputBuffer.peekFirst().getData()[0]);
				}else{
					reading = false;
				}
				if(incomingPacket != null && incomingPacket.isComplete()){
					switch(incomingPacket.getType()){
						case ServCliPacket.update:
							connectionStatus = new String(incomingPacket.getData());
							break;
						case ServCliPacket.ok:
			//				stdio.debugAndLog(this,"Adding packet to buffer");
							InputBuffer.add(incomingPacket);
							incomingPacket = null;
							break;
					}
				}
			}
		}catch(IOException ioe){
			ioe.printStackTrace();
			close = true;
		}
	//derp
	}
	public void close(){
		close = true;
	}
	public boolean isClosed(){
		return close;
	}
	/*
		I'm fairly certain this is thread-safe since InputBuffer is a linked list.

		The only adding we do to it is at the head, and the only removing we do is from the tail.
		No overlap, right?
	*/
	public synchronized LinkedList<AddressedDatum> getPackets(){
		LinkedList<AddressedDatum> returnable = new LinkedList<AddressedDatum>();
	//	if(InputBuffer.size()>0){
	//		System.out.println("Packet size!!!: "+InputBuffer.peekFirst().getSize());
	//		System.out.println("Complete? "+InputBuffer.peekFirst().isComplete());
	//	}
		while(InputBuffer.size()>0 && InputBuffer.peekFirst().isComplete()){
			returnable.add(new AddressedDatum(cliID, InputBuffer.pollFirst().getData()));
		}
		return returnable;
	}
	public synchronized void connect(String IP, int port){
		InetSocketAddress destAddress = null;
		if(sock.isConnected()){
			stdio.debugAndLog(this, "Socket is already in use! Find your own!");
		}else{
			try{
				destAddress = new InetSocketAddress(InetAddress.getByName(IP), port);
			}catch(UnknownHostException uhe){
				stdio.debugAndLog(this, "Unrecognized host: "+IP);
				destAddress = null;
			}
			if(destAddress != null){
				if(port < 1 || port > 65535){
					stdio.debugAndLog(this, "Invalid port! Ports must be between 1 and 65535 (inclusive)");
				}else{
					retryCount = 3;
					while(!sock.isConnected() && retry && retryCount > 0){
						try{
							stdio.debugAndLog(this, "Attemting to connect to "+destAddress.toString());
							connectionStatus = "Connecting..."+(3-retryCount+1);
							sock = new Socket();
							sock.connect(destAddress, timeoutTime);
							stdio.debugAndLog("Connected! Remote address is "+sock.getRemoteSocketAddress());
							remoteAddress = sock.getRemoteSocketAddress().toString();
							localAddress = sock.getLocalAddress().toString();
						}catch(SocketTimeoutException ste){
							retryCount--;
							connectionStatus = "Connection to host timed out.";
							stdio.debugAndLog(this, "Connection to host timed out.");
							if(retry){
								stdio.debugAndLog(this, retryCount+" retries remaining before giveup.");
							}
						}catch(IOException ioe){
							ioe.printStackTrace();
							retryCount--;
							stdio.debugAndLog(this, "Connection to host failed.");
							if(retry){
								stdio.debugAndLog(this, retryCount+" retries remaining before giveup.");
							}
						}

					}
				}
			}
		}
		if(sock.isConnected()){
			try{
				output = new BufferedOutputStream(sock.getOutputStream());
				input = new BufferedInputStream(sock.getInputStream());
			}catch(IOException ioe){
				ioe.printStackTrace();
				close();
			}
			lastValidDest = destAddress;
			write(new ServCliPacket(ServCliPacket.handshake, new byte[0]));
		}
	}
	public void run(){
		while((retry || allowReconnect) && !killed){
			//System.out.println(sock.isConnected());
			while(!close && sock.isConnected()){
				writeBuffer();
				read();
				//System.out.println("Connection read/write");
				try{
					Thread.sleep(1);
				}catch(Exception e){
					e.printStackTrace();
				}
			}
			if(retry && lastValidDest != null){
				connect(lastValidDest.getHostName(), lastValidDest.getPort());
			}
			try{
				Thread.sleep(10);
			}catch(InterruptedException ie){
				/*
					This is a relatively poor and non-loved solution to the problem of multiple connections
					eating all the cpu of the server for no good reason. Ideally there would be some daemon
					which can tell what connections should be awake and wake them as neded. The connections
					themselves could then sleep forever or until a packet needs to be sent.
				*/
			}
		}
		try{
			sock.close();
			input = null;
			output = null;
			stdio.debugAndLog("Connection to "+remoteIP()+" has closed.");
		}catch(IOException ioe){
			ioe.printStackTrace();
		}
		System.out.println("SevCliConn Thread ended.");
	}
	public void killConnection(){
		//Agh you got me.
		killed = true;
		close();
	}
	public boolean connected(){
		return sock.isConnected();
	}
	public synchronized void write(ServCliPacket packet){
		if(sock.isConnected()){
			OutputBuffer.add(packet);
		}else{
			stdio.debugAndLog(this, "Stuff.");
		}
	}
}