/*
	This defines the packets used by a ServCliConn

	ServCliPackets are created with one byte to define what type of packet it is,
	and an array of packets that are the data to the packet.

	When a servCliPacket is converted into an array of bytes to be written,
	it's done so in the order of type, length, data, so that the first recieved
	byte is the type of packet, followed by a pair of bytes to say how long the rest
	of the data should be.
*/

package com.ixee.net;

import com.ixee.io.stdio;
import com.ixee.io.MarkedOutputStream;
import com.ixee.io.MarkedInputStream;
import com.ixee.dataops.BitOperations;

public class ServCliPacket{
	//list of packet types.
	public static final byte ok = 0x03;
	public static final byte update = 0x01;
	public static final byte accept = 0x02;
	public static final byte heartbeat = 0x10;
	public static final byte important = (byte)0xFF;
	public static final byte handshake = 0x0F;
	public static final byte rejection = 0x0E;
	public static final byte undefined = (byte)0x80;
	public static final byte authenticate = 0x0C;
	int size = 0;
	byte packetType = ServCliPacket.undefined;
	MarkedOutputStream mos = new MarkedOutputStream();

	public ServCliPacket(byte type, byte[] data){
		packetType = type;
		size = data.length;
		appendData(data);
	}
	public ServCliPacket(byte type, int sizee){
		packetType = type;
		size = sizee;
	}

	public ServCliPacket(byte[] input){
		MarkedInputStream mis = new MarkedInputStream(input);
		packetType = mis.get();
		size = BitOperations.toInt(mis.get(4));
		appendData(mis.get(input.length-5));
		stdio.debugAndLog("Packet built with size "+size);
	}

	public void appendData(byte[] data){
		//System.out.println("Appending "+data.length+" bytes");
		mos.write(data);
	}

	public byte[] asBytes(){
		MarkedOutputStream output = new MarkedOutputStream();
		output.write(packetType);
		output.write(BitOperations.toByteArray(size));
		output.write(mos.getBuffer());
		return output.getBuffer();
	}
	public byte[] getData(){
		return mos.getBuffer();
	}
	public byte getType(){
		return packetType;
	}
	public int getSize(){
		return size;
	}
	public int getDataSize(){
		return mos.getSize();
	}
	public int getRemaining(){
		return (size-mos.getSize());
	}
	public boolean isComplete(){
		boolean returnable = false;
		if(size == mos.getSize()){
			returnable = true;
		}
		return returnable;
	}
	public String toString(){
		String returnable = "";
		returnable = returnable.concat(packetType+" ");
		returnable = returnable.concat(size+" ");
		returnable = returnable.concat("Filled: "+isComplete());
		return returnable;
	}
}