/*
	This class is intended to exist on its own, simply waiting on some port
	for incoming requests to respond to, which it creates a servCliConn for
	and throws it into the list of connections for some other portion of the
	program to access. It also handles blacklisting certain IPs for whatever reason.
*/

package com.ixee.net;

import java.util.LinkedList;
import java.net.ServerSocket;
import java.util.ArrayList;
import java.io.IOException;
import com.ixee.io.stdio;

public class ServConnSpooler extends Thread{
	ArrayList<ServCliConn> connections = new ArrayList<ServCliConn>();
	volatile ArrayList<String> blacklistedIPs = new ArrayList<String>();
	ServerSocket servSock;
	volatile LinkedList<ServCliConn> newestConnections = new LinkedList<ServCliConn>();
	volatile LinkedList<ServCliConn> disconnections = new LinkedList<ServCliConn>();

	int portToUse = 9631;
	boolean spooling = true;
	int maxConnections = 1;

	public ServConnSpooler(int maxConns){
		maxConnections = maxConns;
		try{
			servSock = new ServerSocket(portToUse, maxConns);
		}catch(IOException ioe){
			ioe.printStackTrace();
		}
	}
	public ServConnSpooler(int port, int maxConns){
		maxConnections = maxConns;
		portToUse = port;
		try{
			servSock = new ServerSocket(portToUse, maxConns);
		}catch(IOException ioe){
			ioe.printStackTrace();
		}
	}
	public void setMaxConnections(int i){
		maxConnections = i;
	}
	public synchronized ArrayList<ServCliConn> getConnections(){
		checkConnectionsForLife();
		return connections;
	}
	public int getPort(){
		return servSock.getLocalPort();
	}
	public synchronized void checkConnectionsForLife(){
		for(int i=0;i<connections.size();i++){
			if(connections.get(i).isClosed()){
				disconnections.add(connections.get(i));
				connections.get(i).killConnection();
				connections.remove(connections.get(i));
			}
		}
	}
	public void run(){
		while(spooling){
			if(connections.size() < maxConnections){
				try{
					connections.add(new ServCliConn(servSock.accept()));
					if(isBlaclisted(connections.get(connections.size()-1).getIP())){
						stdio.debugAndLog(this, "Rejected blacklisted IP:"+ connections.get(connections.size()-1).getIP());
						connections.remove(connections.get(connections.size()-1));
					}else{
						connections.get(connections.size()-1).start();
						newestConnections.add(connections.get(connections.size()-1));
					}
				}catch(IOException ioe){
					ioe.printStackTrace();
				}
			}else{
				try{
					Thread.sleep(250);
				}catch(InterruptedException ie){
					ie.printStackTrace();
				}
			}
			try{
				Thread.sleep(1);
			}catch(InterruptedException ie){
			}
		}
	}
	public void end(){
		spooling = false;
	}
	public synchronized void blacklist(ServCliConn conn){
		blacklistedIPs.add(conn.getIP());
		conn.close();
		connections.remove(conn);
		stdio.debugAndLog("Blacklisted!"+conn.getIP());
	}
	private synchronized boolean isBlaclisted(String IP){
		boolean returnable = false;
		for(int i=0;i<blacklistedIPs.size();i++){
			if(blacklistedIPs.get(i).equals(IP)){
				returnable = true;
			}
		}
		return returnable;
	}
	public String getAddress(){
		return servSock.getInetAddress().toString().concat(Integer.toString(servSock.getLocalPort()));
	}
	/*
		This method needs to be rewritten to return a nice linkedList of ServCliCon that have connected since the last check.
		Right now this is just bad practice and is liable to miss connections.
	*/
	public synchronized LinkedList<ServCliConn> getNewest(){
		LinkedList<ServCliConn> returnable = new LinkedList<ServCliConn>();
		while(newestConnections.size()>0){
			returnable.add(newestConnections.pollLast());
		}
		return returnable;
	}
	public synchronized LinkedList<ServCliConn> getDroppedConnections(){
		checkConnectionsForLife();
		LinkedList<ServCliConn> returnable = new LinkedList<ServCliConn>();
		while(disconnections.size()>0){
			returnable.add(disconnections.pollLast());
		}
		return returnable;
	}
}