//TODO: Make it so that when PhysObjectWrappers are added to a TreeNode, their positions are sorted for another pass of broad detection based on positions.

/*
	TreeNode subdivisions:
		[1, 3]
		[0, 2]
*/
package com.ixee.phys;

import com.ixee.datastruct.Coordinate;
import java.util.Set;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Random;

public class CollisionQuadTree{
	//Now have we broken sceg?
	int numObjects;
	HashMap<String, TreeNode> leaves;
	HashMap<PhysObject, PhysObjectWrapper[]> objects;
	TreeNode root;

	public static void main(String[] args){
		int numObjects = 10000;
		int numRefreshes = 100;
		Random rand = new Random(0);
		System.out.println("~~~*~*~*~CollisionQuadtree Test~*~*~*~~");
		CollisionQuadTree tree = new CollisionQuadTree(8, 10, new Coordinate(-2048, -2048), new Coordinate(2048, 2048));
		for(int i=0;i<numObjects;i++){
			tree.add(new PhysObject(new Coordinate((rand.nextDouble()-0.5)*2048, (rand.nextDouble()-0.5)*2048)));
		}
		double d = System.nanoTime();

		System.out.println("Building tree...");
		d = System.nanoTime();
		tree = new CollisionQuadTree(8, 10, new Coordinate(-2048, -2048), new Coordinate(2048, 2048));
		for(int i=0;i<numObjects;i++){
			tree.add(new PhysObject(new Coordinate((rand.nextDouble()-0.5)*2048, (rand.nextDouble()-0.5)*2048)));
		}
		System.out.println(" tree completed in "+(System.nanoTime()-d)/1000000+"ms");

		//TreeNode[] leafList = tree.getLeaves();

		System.out.println("Refreshing......."+numRefreshes+" times.");
		for(int i=0;i<numRefreshes;i++){
			System.out.print(" refresh "+i+": ");
			d = System.nanoTime();
			tree.refresh();
			System.out.println((System.nanoTime()-d)/1000000+"ms");
		}
	}

	public CollisionQuadTree(int mDepth, int divThreshold, Coordinate bottomLeft, Coordinate topRight){
		numObjects = 0;
		root = new TreeNode(this, null, mDepth, divThreshold, bottomLeft, topRight, (byte)-1);
		objects = new HashMap<PhysObject, PhysObjectWrapper[]>(1024);
		leaves = new HashMap<String, TreeNode>(512);
		leaves.put(root.toString(), root);
	}
	public PhysObject[][] getCollidablePairs(){
		//System.out.println(numObjects + " items to process");
		TreeNode[] leafList = getLeaves();
		PhysObject[][] returnable = new PhysObject[leafList.length][];
		//System.out.println("Shaking "+leafList.length+" leaves for objects."+numObjects+" objects stored in theory.");
		int numThings = 0;
		for(int i=0;i<leafList.length;i++){
			returnable[i] = leafList[i].getContents().toArray(new PhysObject[leafList[i].getContents().size()]);
			numThings += returnable[i].length;
		}
		//System.out.println(numThings+" things fell out.");
		return returnable;
	}
	public void refresh(){
		TreeNode[] leafList = leaves.values().toArray(new TreeNode[leaves.values().size()]);
		double time = 0;
		double temp = 0;
		for(int i=0;i<leafList.length;i++){
			temp = System.nanoTime();
			PhysObjectWrapper[][] wrappers = leafList[i].getWrappers();
			time += System.nanoTime()-temp;
			for(int j=0;j<wrappers.length;j++){
				for(int k=0;k<4;k++){
					if(wrappers[j][k] != null){
						leafList[i].refresh(wrappers[j][k]);
					}
				}
			}
		}
		System.out.println("'"+time/1000000+"'");
	}
		//PhysObjectWrapper[][] objectWrappers = objects.values().toArray(new PhysObjectWrapper[objects.values().size()]);
		//for(int i=0;i<

	public void add(PhysObject o){
		numObjects++;
		PhysObjectWrapper[] toAdd = new PhysObjectWrapper[4];
		toAdd[0] = new PhysObjectWrapper(o, new Coordinate(-o.getRadius(), -o.getRadius()),0);
		toAdd[1] = new PhysObjectWrapper(o, new Coordinate(-o.getRadius(), o.getRadius()),1);
		toAdd[2] = new PhysObjectWrapper(o, new Coordinate(o.getRadius(), -o.getRadius()),2);
		toAdd[3] = new PhysObjectWrapper(o, new Coordinate(o.getRadius(), o.getRadius()),3);
		for(int i=0;i<toAdd.length;i++){
			root.add(toAdd[i]);
		}
		//PhysObjectWrapper[] toAdd = new PhysObjectWrapper[1];
		//toAdd[0] = new PhysObjectWrapper(o, new Coordinate(0,0));
		//root.add(toAdd[0]);
		objects.put(o, toAdd);
		//root.add(o);
	}
	protected void add(PhysObjectWrapper o){
		root.add(o);
	}
	public void remove(PhysObject o){
		numObjects--;
		//PhysObjectWrapper[] toRemove = objects.get(o);
		//if(toRemove != null){
		//	for(int i=0;i<toRemove.length;i++){
		//		remove(toRemove[i]);
		//	}
		//}
		//System.out.println("deleting "+o);
		root.delete(o);
		objects.remove(o);
	}
	protected void remove(PhysObjectWrapper o){
		root.remove(o);
	}
	public void removeLeaf(TreeNode node){
		if(root == node){
		//	System.out.println("Uh oh, why are we removing the root node? Not allowed.");
		}else{
			leaves.remove(node.toString());
		}
	}
	public void addLeaf(TreeNode node){
		leaves.put(node.toString(), node);
	}
	public TreeNode[] getLeaves(){
		return leaves.values().toArray(new TreeNode[leaves.values().size()]);
	}
	public boolean contains(PhysObject po){
		return objects.get(po) != null;
	}
	//public ArrayList<PhysObject> objectsThatLieOn(Coordinate lineStart, Coordinate lineEnd){
	//	ArrayList<PhysObject> returnable = new ArrayList<PhysObject>();
	//	TreeNode currBox = leafThatContains(lineStart);
	//	double slope =
	//	while(currBox.contains(lineEnd) == false){

	//}
}
