package com.ixee.phys;

import com.ixee.datastruct.Coordinate;
import java.util.ArrayList;
import com.ixee.io.stdio;

public class PhysObject{

	//Flags!
	public final static int UNINITIALIZED = -1;
	public final static int READY = 0;

	public final static int POSITION = 0;
	public final static int VELOCITY = 1;
	public final static int ACCELERATION = 2;

	public final static int ROT_POSITION = 3;
	public final static int ROT_VELOCITY = 4;
	public final static int ROT_ACCELERATION = 5;


	//Data!


	// Coordinate position;					//flagged 0
	// Coordinate velocity;					//flagged 1
	// Coordinate acceleration;				//flagged 2
	// Coordinate orientation;				//flagged 3
	// Coordinate rotationalVelocity;		//flagged 4
	// Coordinate rotationalAcceleration;	//flagged 5

	public Coordinate[] positionComponents = new Coordinate[3];
	public Coordinate[] angularComponents = new Coordinate[3];

	//public Coordinate momentOfInertia = new Coordinate(0,0,0);			//for now this and inertia are unused.
	//public double inertia;
	boolean updateState=false;
	double lastUpdateTime=0;

	double deltaT = 0;

	protected double radius;

	int objectType = UNINITIALIZED;

	protected ArrayList<CollisionResolutionRequest> warnings;

	Physics phys;

	public PhysObject(){
		warnings = new ArrayList<CollisionResolutionRequest>();
		for(int i=0;i<3;i++){
			positionComponents[i] = new Coordinate(0,0,0);
			angularComponents[i] = new Coordinate(0,0,0);
		}
		updateState = false;
	}
	public PhysObject(Coordinate c){
		this();
		positionComponents[0] = c;
	}
	public void setRadius(double r){
		radius = r;
	}
	public double getRadius(){
		return radius;
	}
	public void setPhysics(Physics p){
		phys = p;
		if(phys!=null){
			lastUpdateTime = phys.currentTime();
		}
	}
	public PhysObject(Coordinate[] cartesianValues, Coordinate[] angularValues){
		positionComponents = cartesianValues;
		angularComponents = angularValues;

		updateState = false;
	}
	public void addWarning(CollisionResolutionRequest crr){
		warnings.add(crr);
	}
	public ArrayList<CollisionResolutionRequest> getWarnings(){
		return warnings;
	}
	public Coordinate getAttrib(int flag){
		Coordinate returnable = null;
		if(flag>2){
			flag = flag-3;
			returnable = angularComponents[flag];
		}else{
			returnable = positionComponents[flag];
		}
		return returnable;
	}
	public void update(){
		if(phys != null){
			deltaT = (phys.currentTime()-lastUpdateTime)/1000;
			positionComponents[1] = positionComponents[1].add(positionComponents[2].mult(deltaT));
			positionComponents[0] = positionComponents[0].add(positionComponents[1].mult(deltaT));
			angularComponents[1] = angularComponents[1].add(angularComponents[2].mult(deltaT));
			angularComponents[0] = angularComponents[0].add(angularComponents[1].mult(deltaT));
			//------------------------------------------------------------------
			//*******************Feb 15, 2012. Never forget.********************
			//------------------------------------------------------------------
			//for(int i=2;i>0;i--){				//for i = 2 -> 1
			//	for(int j=0;j<3;j++){				//for j = 0 -> 2
			//		positionComponents[i-1] = incrimentAttrib(j, i-1, getAttrib(i).getCoords(j)*deltaT);	//incriment value (j) of coordinate(i-1) by coordinate(i)*deltaT
			//		angularComponents[i-1] = incrimentAttrib(j, i-1+3, getAttrib(i+3).getCoords(j)*deltaT);
			//	}
			//}
			angularComponents[0].setX(PhysMath.reduceAngle(angularComponents[0].getX()));
			lastUpdateTime = phys.currentTime();
		}else{
			stdio.debugAndLog(this, "Warning, "+this+" is not a member of a Physics engine");
		}
	}
	public void setAttrib(int flag, Coordinate value){
		if(flag>2){
			angularComponents[flag-3] = value;
		}else{
			positionComponents[flag] = value;
		}
	}
	public boolean getUpdateState(){
		return updateState;
	}
	public void setUpdateState(boolean newState){
		updateState = newState;
	}
	public PhysObject makeClone(){
		PhysObject returnable = new PhysObject();
		for(int i=0;i<3;i++){
			returnable.positionComponents[i] = new Coordinate(positionComponents[i]);
			returnable.angularComponents[i] = new Coordinate(angularComponents[i]);
		}
		return returnable;
	}
	public Coordinate getAngularPosition(){
		return angularComponents[0];
	}
	public Coordinate getAngularVelocity(){
		return angularComponents[1];
	}
	public Coordinate getAngularAcceleration(){
		return angularComponents[2];
	}
	public Coordinate getPosition(){
		return positionComponents[0];
	}
	public Coordinate getVelocity(){
		return positionComponents[1];
	}
	public Coordinate getRotPosition(){
		return angularComponents[0];
	}
	public Coordinate getRotVelocity(){
		return angularComponents[1];
	}
	public void setPosition(Coordinate c){
		positionComponents[0] = c;
	}
	public void setVelocity(Coordinate c){
		positionComponents[1] = c;
	}
	public void setAcceleration(Coordinate c){
		positionComponents[2] = c;
	}
//	public GeomLine getVelocityPath(){
//		return new GeomLine(positionComponents[POSITION], new Vect(positionComponents[VELOCITY]));
//	}
}