package com.ixee.phys;

import com.ixee.datastruct.Coordinate;

public class Physics{
	double currTime = 0;
	CollisionQuadTree gridBacking;
	public Physics(){
	}
	public void initQuadTree(int depth, int objThreshold, double radius){
		gridBacking = new CollisionQuadTree(depth, objThreshold, new Coordinate(-radius, -radius), new Coordinate(radius, radius));
	}
	public void add(PhysObject o){
		if(gridBacking != null){
			gridBacking.add(o);
		}
		o.setPhysics(this);
	}
	public void remove(PhysObject o){
		if(gridBacking != null){
			gridBacking.remove(o);
		}
		o.setPhysics(null);
	}
	public void refresh(){
		gridBacking.refresh();
	}
	public TreeNode[] getLeaves(){
		return gridBacking.getLeaves();
	}
	public PhysObject[][] getCollidablePairs(){
		return gridBacking.getCollidablePairs();
	}
	public void processCollisions(){
		PhysObject[][] collidables = gridBacking.getCollidablePairs();
		//System.out.println(collidables.length+" many grids to process");
		for(int i=0;i<collidables.length;i++){
			for(int j=0;j<collidables[i].length;j++){
				for(int k=j+1;k<collidables[i].length;k++){
					if(PhysMath.pythag(collidables[i][j].getPosition(), collidables[i][k].getPosition()) < collidables[i][j].getRadius()+collidables[i][k].getRadius()){
						collidables[i][j].addWarning(new CollisionResolutionRequest(collidables[i][j], collidables[i][k]));
						collidables[i][k].addWarning(new CollisionResolutionRequest(collidables[i][k], collidables[i][j]));
					}
				}
			}
		}
	}
	public boolean contains(PhysObject po){
		return gridBacking.contains(po);
	}
	public void setTime(double newTime){
		currTime = newTime;
	}
	public double currentTime(){
		return currTime;
	}
	public static double magnitude(Coordinate c){
		return Math.sqrt(c.getX()*c.getX()+c.getY()*c.getY());
	}
	public static double angleToIntercept(Coordinate c, double sourceSpeed, Coordinate di, Coordinate dv){
		double toASin = (magnitude(dv)*Math.sin(Math.atan2(dv.getY(), dv.getX()))/sourceSpeed);
		double returnable = Math.asin(toASin);
		if(dv.getY() == 0){
			if(dv.getX() >= 0){
				returnable = 0;
			}else{
				returnable = Math.PI;
			}
		}
		Coordinate delta = di.sub(c);
		double baseToAddTo = Math.atan2(delta.getY(), delta.getX());
		if(delta.getY() != 0){
			returnable += baseToAddTo;
		}
		return returnable;
	}
	public static double angleOf(Coordinate c){
		return Math.atan2(c.getY(), c.getX());
	}
	public static double timeToIntercept(Coordinate c, double sourceSpeed, Coordinate di, Coordinate dv){
		double angle = angleToIntercept(c, sourceSpeed, di, dv);
		double dist = pythag(c, di);
		double angleC = 180-angle-angleOf(dv);
		double relativeVelocity = Math.sin(angleC)*magnitude(dv)/Math.sin(angle);
		return dist/relativeVelocity;
	}
	public static double pythag(Coordinate a, Coordinate b){
		return pythag(a.sub(b));
	}
	public static double pythag(Coordinate c){
		return Math.sqrt(c.getX()*c.getX()+c.getY()*c.getY());
	}

}