package com.ixee.phys;

import com.ixee.datastruct.Coordinate;
import com.ixee.io.stdio;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;

public class TreeNode{
	//HashMap<PhysObjectWrapper, PhysObject> nodeMembers;
	HashMap<PhysObject, PhysObjectWrapper[]> nodeWrappers;
	TreeNode[] branches;
	boolean branched;
	Coordinate bottomLeft;
	Coordinate topRight;
	Coordinate center;
	int depth;
	int divisionThreshold;
	int containedItems;
	int maxDepth;
	double size;
	byte position = 4;
	TreeNode parent;
	CollisionQuadTree tree;
	ArrayList<PhysObject> items;


	public TreeNode(CollisionQuadTree treeBoss, TreeNode p, int md, int division, Coordinate bl, Coordinate tr, byte positionToParent){
		position = positionToParent;
		items = new ArrayList<PhysObject>();
		//nodeMembers = new HashMap<PhysObjectWrapper, PhysObject>(division*4);
		nodeWrappers = new HashMap<PhysObject, PhysObjectWrapper[]>(division*4);
		tree = treeBoss;
		containedItems = 0;
		branches = null;
		branched = false;
		bottomLeft = bl;
		topRight = tr;
		parent = p;
		maxDepth = md;
		divisionThreshold = division;
		if(divisionThreshold <= 1){
			divisionThreshold = 2;
		}
		if(p == null){
			depth = 0;
		}else{
			depth = p.getDepth()+1;
		}
		center = (bottomLeft.add(topRight)).mult(0.5);
	}
	public void subdivide(){
		//System.out.println("REBRANCHING.");
		if(branches == null){
			branches = new TreeNode[4];
			branches[0] = new TreeNode(tree, this, maxDepth, divisionThreshold, bottomLeft, center, (byte)0);
			branches[1] = new TreeNode(tree, this, maxDepth, divisionThreshold, new Coordinate(bottomLeft.getX(), center.getY()), new Coordinate(center.getX(), topRight.getY()), (byte)0x1);
			branches[2] = new TreeNode(tree, this, maxDepth, divisionThreshold, new Coordinate(center.getX(), bottomLeft.getY()), new Coordinate(topRight.getX(), center.getY()), (byte)0x2);
			branches[3] = new TreeNode(tree, this, maxDepth, divisionThreshold, center, topRight, (byte)0x3);
		}else{
			for(int i=0;i<branches.length;i++){
				if(branches[i].branched()){
					branches[i].clear();
				}
			}
		}
		tree.removeLeaf(this);
		for(int i=0;i<branches.length;i++){
			tree.addLeaf(branches[i]);
		}
		Iterator<PhysObjectWrapper[]> objects = nodeWrappers.values().iterator();
		PhysObjectWrapper[] currentWrap;

		while(objects.hasNext()){
			currentWrap = objects.next();
			for(int i=0;i<4;i++){
				if(currentWrap[i] != null){
					PhysObjectWrapper current = currentWrap[i];
					Coordinate position = current.getPosition();
					if(position.getX() < center.getX()){
						if(position.getY() < center.getY()){
							branches[0].add(current);
						}else{
							branches[1].add(current);
						}
					}else{
						if(position.getY() < center.getY()){
							branches[2].add(current);
						}else{
							branches[3].add(current);
						}
					}
				}
			}
		}

		branched = true;
	}
	public void reconstitute(){
		branched = false;
		//System.out.println("RECONSTITUTING.");
		for(int i=0;i<branches.length;i++){
			tree.removeLeaf(branches[i]);
			branches[i].clear();
		}
	//	parent.tryBurning();
		if(parent == null || parent.branched()){
			tree.addLeaf(this);
		}
	}
	public LinkedList<Byte> addressOf(){
		LinkedList<Byte> addressList;
		if(parent != null){
			addressList = parent.addressOf();
		}else{
			addressList = new LinkedList<Byte>();
		}
		addressList.add(new Byte(position));
		return addressList;
	}
	public boolean branched(){
		return branched;
	}
	public int getDepth(){
		return depth;
	}
	public void clear(){
		nodeWrappers.clear();
		containedItems = 0;
		items.clear();
		//ArrayList<TreeNode> nodesToReconstitute =
		if(branched){
			reconstitute();
		}
	}
	//public void
	public void tryBurning(){
	//	if(containedItems < divisionThreshold && branched && depth != maxDepth){
	//		reconstitute();
	//	}
	}
	public void refresh(PhysObjectWrapper o){
		if(!this.canContain(o.getPosition())){
			if(nodeWrappers.get(o.contents) != null){
			//	System.out.println("Refresh determined that "+o.contents+" cannot be contained in "+this);
				PhysObjectWrapper[] wrappers = nodeWrappers.get(o.contents);
				wrappers[o.index] = null;
				boolean anyLeft = false;
				for(int i=0;i<wrappers.length;i++){
					if(wrappers[i] != null){
						anyLeft = true;
					}
				}
				if(!anyLeft){
					nodeWrappers.remove(o.contents);
					items.remove(items.indexOf(o.contents));
				}
				containedItems = items.size();
				if(containedItems < divisionThreshold && branched && depth != maxDepth){
					reconstitute();
				}
				if(parent != null){
					parent.refresh(o);
				}
			}
		}else{
		//	if(nodeWrappers.get(o.contents) != null){
		//		remove(o);
			//	containedItems--;
		//	}
			//	System.out.println("Refresh determined that "+o.contents+" contains "+this);
		//	add(o);
		}
	}
	public void add(PhysObjectWrapper o){
		//System.out.println("ADD");
		if(this.canContain(o.getPosition())){
			if(nodeWrappers.get(o.contents) == null){
				nodeWrappers.put(o.contents, new PhysObjectWrapper[4]);
				items.add(o.contents);
			}
			nodeWrappers.get(o.contents)[o.index] = o;

			containedItems = items.size();
			if(!branched && containedItems >= divisionThreshold && depth != maxDepth){
				subdivide();
			}
			if(branched){
				Coordinate position = o.getPosition();
				if(position.getX() < center.getX()){
					if(position.getY() < center.getY()){
						branches[0].add(o);
					}else{
						branches[1].add(o);
					}
				}else{
					if(position.getY() < center.getY()){
						branches[2].add(o);
					}else{
						branches[3].add(o);
					}
				}
			}
		}
	}
	public boolean canContain(Coordinate pos){
		boolean returnable = false;
		//Coordinate pos = o.getPosition();
		if(pos.getX()>bottomLeft.getX() && pos.getX() < topRight.getX()){
			if(pos.getY()>bottomLeft.getY() && pos.getY() < topRight.getY()){
				returnable = true;
			}
		}
		return returnable;
	}
	public void remove(PhysObjectWrapper o){
		if(nodeWrappers.get(o.contents) != null){//then this node contains o.contents
			PhysObjectWrapper[] wrappers = nodeWrappers.get(o.contents);
			wrappers[o.index] = null;
			boolean anyLeft = false;
			for(int i=0;i<wrappers.length;i++){
				if(wrappers[i] != null){
					anyLeft = true;
				}
			}
			if(!anyLeft){
				nodeWrappers.remove(o.contents);
				items.remove(items.indexOf(o.contents));
				containedItems--;
			}
			if(containedItems < divisionThreshold && branched && depth != maxDepth){
				reconstitute();
			}
			//if(parent != null){
			//	parent.remove(o);
			//}//else{
			//	stdio.debugAndLog(this, o+" had a wrapper removed, position at "+o.getPosition()+". Invesitgate.");
			//}
			if(branched){
				for(int i=0;i<4;i++){
					if(branches[i].contains(o.contents)){
						branches[i].remove(o);
					}
				}
			}
		}
	}
	public void delete(PhysObject o){
		if(nodeWrappers.get(o) != null){
			nodeWrappers.remove(o);
			items.remove(items.indexOf(o));
			containedItems--;
			if(branched){
				for(int i=0;i<4;i++){
					if(branches[i].contains(o)){
						branches[i].delete(o);
					}
				}
			}
			if(containedItems < divisionThreshold && branched && depth != maxDepth){
				reconstitute();
			}
		}
	}
	public boolean contains(PhysObject o){
		return nodeWrappers.get(o) != null;
	}
	public PhysObjectWrapper[][] getWrappers(){
		return nodeWrappers.values().toArray(new PhysObjectWrapper[nodeWrappers.values().size()][]);
	}
	public ArrayList<PhysObject> getContents(){
		return items;
	}
	//public PhysObject[] getContents(){
		//Iterator<PhysObject> iterator = nodeMembers.values().iterator();
		//HashSet<PhysObject> returnableSet = new HashSet<PhysObject>();
		//while(iterator.hasNext()){
		//	returnableSet.add(iterator.next());
		//}
		//return returnableSet.toArray(new PhysObject[returnableSet.size()]);
		//return nodeMembers.values().toArray(new PhysObject[nodeMembers.values().size()]);
	//}
	public Coordinate bottomLeft(){
		return bottomLeft;
	}
	public Coordinate topRight(){
		return topRight;
	}
}