package com.ixee.ui.ogl;

import com.ixee.io.stdio;
import java.awt.Color;
import com.ixee.GraphIX.glEEngine;
import com.ixee.datastruct.Coordinate;

public class UIText extends UIElement{
	public static final byte H_LEFT = 0x01;
	public static final byte H_RIGHT = 0x02;
	public static final byte H_CENTER = 0x03;
	public static final byte V_TOP = 0x10;
	public static final byte V_BOTTOM = 0x20;
	public static final byte V_CENTER = 0x30;
	public static final byte B_LEFT = (byte)(H_LEFT|V_BOTTOM);
	public static final byte B_RIGHT = (byte)(H_RIGHT|V_BOTTOM);
	public static final byte T_LEFT = (byte)(H_LEFT|V_TOP);
	public static final byte T_RIGHT = (byte)(H_RIGHT|V_TOP);

	static final int V_MASK = 0x000000F0;
	static final int H_MASK = 0x0000000F;

	Color textColor = new Color(1f,1f,1f);
	int centering;
	String text;

	int width;
	int height;
	int maxChars = 900;

	boolean textChanged = false;

	public UIText(Coordinate loc, String tx, byte positioning, glEEngine e){
		this(loc,tx,positioning);
	}
	public UIText(Coordinate loc, String tx, byte positioning){
		this(loc, tx, positioning, new Color(1f, 1f, 1f));
	}
	public UIText(Coordinate loc, String tx, byte positioning, Color c){
		super(loc);
		textColor = c;
		centering = 0x00 | positioning;
		setText(tx);
	}
	public UIText(Coordinate loc, String tx, byte positioning, glEEngine e, Color c){
		this(loc,tx,positioning,c);
	}
	public boolean contains(Coordinate c){
		boolean returnable = false;
		if(Math.abs(centerpoint.getX()-c.getX()) < width && Math.abs(centerpoint.getY()-c.getY()) < height){
			returnable = true;
		}
		return returnable;
	}
	public void setCenter(byte b){
		centering = b;
	}
	public void setText(String s, glEEngine e){
		setText(s);
	}
	public void setText(String s){
		text = s;
		textChanged = true;
	}
	public void setNumChars(int chars){
		maxChars = chars+1;
	}
	public void draw(glEEngine e){
		if(textChanged){
			height = e.getFont().getHeight();
			width = e.getFont().getWidth(text.substring(0, Math.min(maxChars, text.length())));
			textChanged = false;
		}
		int V_POS = V_MASK & centering;
		int H_POS = H_MASK & centering;

		int w_off = width;
		int h_off = height;
		if(H_POS == H_LEFT){
			w_off = width/2;
		}else if(H_POS == H_RIGHT){
			w_off = -width/2;
		}else{
			w_off = 0;
		}
		if(V_POS == V_TOP){
			h_off = height/2;
		}else if(V_POS == V_BOTTOM){
			h_off = -height/2;
		}else{
			h_off = 0;
		}

		e.drawString(text.substring(0, Math.min(maxChars, text.length())), centerpoint.getX()+w_off, centerpoint.getY()+h_off, textColor);
	}
}

