/*
	This class implements a LinkedList (almost) that works in two directions.
	This exists mostly for things such as images where we traverse both up/down and left/right.
	The reason this is (almost) a linkedlist is because it contains an ArrayList
	of all its contents, so it's more of a simulated LinkedList than anything else.
*/

package com.ixee.util;

import java.util.ArrayList;

public class DoublyLinkedList<T extends ListableItem>{
	ArrayList<T> contents;
	ListableItem first = null;
	ListableItem last = null;
	int width;
	int height;
	int iter = 0;
	public static final int LEFT = 0;
	public static final int RIGHT = 1;
	public static final int DOWN = 2;
	public static final int UP = 3;

	public DoublyLinkedList(int sizex, int sizey){
		contents = new ArrayList<T>();
		for(int i=0;i<sizex*sizey;i++){
			contents.add(null);
		}
		System.out.println("DoublyLinkedList: contents size is "+sizex*sizey);
		width = sizex;
		height = sizey;
	}
	public T get(int index){
		return contents.get(index);
	}
	public void add(T item){
		contents.set(iter, item);
		item.setIndex(iter);
		iter++;
	}
	public void insert(T item){
		//Avoid using this, it's sort of really slow
		//do magic to copy all contents past index i over one position.
		//return it to a list format
		//done
	}
	public void remove(T item){
		contents.set(contents.indexOf(item), null);
	}
	public T getLinkedItem(ListableItem item, int direction){
		T returnable = null;
		int indexOfTarget = -1;
		indexOfTarget = getIndex(item, direction);
		if(indexOfTarget != -1 && contents.get(indexOfTarget) != null){
			returnable = contents.get(indexOfTarget);
		}
		return returnable;
	}
	public int getIndex(ListableItem item, int direction){
		int returnable = item.getIndex();
		switch(direction){
			case LEFT:
				if(returnable%width != 0){
					returnable--;
				}else{
					returnable = -1;
				}
				break;
			case RIGHT:
				if(returnable%width != width-1){
					returnable++;
				}else{
					returnable = -1;
				}
				break;
			case UP:
				if(returnable + width < contents.size()){
					returnable = returnable+width;
				}else{
					returnable = -1;
				}
				break;
			case DOWN:
				if(returnable - width >= 0){
					returnable = returnable - width;
				}else{
					returnable = -1;
				}
				break;
			default:
				returnable = -1;
				break;
		}
		return returnable;
	}
	public T getItem(int index){
		return contents.get(index);
	}
	public T getItem(int x, int y){
		return contents.get(width*y+x);
	}
	public int size(){
		return contents.size();
	}
}